import weakref
from _typeshed import Incomplete, Self, SupportsItems, SupportsKeysAndGetItem
from collections.abc import Callable, Generator, Hashable, Iterable, Iterator, Mapping
from typing import Any, Generic, TypeVar, overload

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_T = TypeVar("_T")

PREV: int
NEXT: int
KEY: int
VALUE: int
DEFAULT_MAX_SIZE: int

class LRI(dict[_KT, _VT]):
    hit_count: int
    miss_count: int
    soft_miss_count: int
    max_size: int
    on_miss: Callable[[_KT], _VT]
    def __init__(self, max_size: int = 128, values: Incomplete | None = None, on_miss: Incomplete | None = None) -> None: ...
    def __setitem__(self, key: _KT, value: _VT) -> None: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    @overload
    def get(self, key: _KT, default: None = None) -> _VT | None: ...
    @overload
    def get(self, key: _KT, default: _T) -> _T | _VT: ...
    def __delitem__(self, key: _KT) -> None: ...
    @overload
    def pop(self, key: _KT) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _T) -> _T | _VT: ...
    def popitem(self) -> tuple[_KT, _VT]: ...
    def clear(self) -> None: ...
    def copy(self: Self) -> Self: ...
    @overload
    def setdefault(self, key: _KT, default: None = None) -> _VT: ...
    @overload
    def setdefault(self, key: _KT, default: _VT) -> _VT: ...
    def update(self, E: SupportsKeysAndGetItem[_KT, _VT] | Iterable[tuple[_KT, _VT]], **F: _VT) -> None: ...  # type: ignore[override]

class LRU(LRI[_KT, _VT]):
    def __getitem__(self, key: _KT) -> _VT: ...

def make_cache_key(
    args: Iterable[Hashable],
    kwargs: SupportsItems[Hashable, Hashable],
    typed: bool = False,
    kwarg_mark: object = ...,
    fasttypes: frozenset[type] = ...,
): ...

class CachedFunction:
    func: Incomplete
    get_cache: Incomplete
    scoped: Incomplete
    typed: Incomplete
    key_func: Incomplete
    def __init__(self, func, cache, scoped: bool = True, typed: bool = False, key: Incomplete | None = None): ...
    def __call__(self, *args, **kwargs): ...

class CachedMethod:
    func: Incomplete
    get_cache: Incomplete
    scoped: Incomplete
    typed: Incomplete
    key_func: Incomplete
    bound_to: Incomplete
    def __init__(self, func, cache, scoped: bool = True, typed: bool = False, key: Incomplete | None = None): ...
    def __get__(self, obj, objtype: Incomplete | None = None): ...
    def __call__(self, *args, **kwargs): ...

def cached(cache: Mapping[Any, Any], scoped: bool = True, typed: bool = False, key: Incomplete | None = None): ...
def cachedmethod(cache, scoped: bool = True, typed: bool = False, key: Incomplete | None = None): ...

class cachedproperty(Generic[_T]):
    func: Callable[[Incomplete], _T]
    def __init__(self, func: Callable[[Incomplete], _T]) -> None: ...
    def __get__(self, obj: _T, objtype: type | None = None): ...

class ThresholdCounter(Generic[_T]):
    total: int
    def __init__(self, threshold: float = 0.001) -> None: ...
    @property
    def threshold(self) -> float: ...
    def add(self, key: _T) -> None: ...
    def elements(self) -> Iterator[_T]: ...
    def most_common(self, n: int | None = None) -> list[tuple[_T, int]]: ...
    def get_common_count(self) -> int: ...
    def get_uncommon_count(self) -> int: ...
    def get_commonality(self) -> float: ...
    def __getitem__(self, key: _T) -> int: ...
    def __len__(self) -> int: ...
    def __contains__(self, key: _T) -> bool: ...
    def iterkeys(self) -> Iterator[_T]: ...
    def keys(self) -> list[_T]: ...
    def itervalues(self) -> Generator[int, None, None]: ...
    def values(self) -> list[int]: ...
    def iteritems(self) -> Generator[tuple[_T, int], None, None]: ...
    def items(self) -> list[tuple[_T, int]]: ...
    def get(self, key: _T, default: int = 0) -> int: ...
    def update(self, iterable: Iterable[_T] | Mapping[_T, int], **kwargs: Iterable[_T] | Mapping[_T, int]) -> None: ...

class MinIDMap(Generic[_T]):
    mapping: weakref.WeakKeyDictionary[_T, int]
    ref_map: dict[_T, int]
    free: list[int]
    def __init__(self) -> None: ...
    def get(self, a: _T) -> int: ...
    def drop(self, a: _T) -> None: ...
    def __contains__(self, a: _T) -> bool: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __len__(self) -> int: ...
    def iteritems(self) -> Iterator[tuple[_T, int]]: ...



from _typeshed import Incomplete

def main() -> None: ...
def run_command(command_line) -> None: ...
def print_formatted_length(value) -> None: ...
def print_formatted_number(value) -> None: ...
def print_formatted_size(value, binary) -> None: ...
def print_formatted_table(delimiter) -> None: ...
def print_formatted_timespan(value) -> None: ...
def print_parsed_length(value) -> None: ...
def print_parsed_size(value) -> None: ...
def demonstrate_ansi_formatting() -> None: ...
def demonstrate_256_colors(i, j, group: Incomplete | None = None) -> None: ...



import ctypes
from _typeshed import StrPath
from collections.abc import Sequence

# At runtime, Query and Range are available only in tree_sitter.binding
from tree_sitter.binding import Node as Node, Parser as Parser, Query, Tree as Tree, TreeCursor as TreeCursor

class Language:
    @staticmethod
    def build_library(output_path: str, repo_paths: Sequence[StrPath]) -> bool: ...
    name: str
    lib: ctypes.CDLL
    language_id: int
    # library_path is passed into ctypes LoadLibrary
    def __init__(self, library_path: str, name: str) -> None: ...
    def field_id_for_name(self, name: str) -> int | None: ...
    def query(self, source: str) -> Query: ...



from html.parser import HTMLParser

def diagnose(data) -> None: ...
def lxml_trace(data, html: bool = True, **kwargs) -> None: ...

class AnnouncingParser(HTMLParser):
    def handle_starttag(self, name, attrs) -> None: ...
    def handle_endtag(self, name) -> None: ...
    def handle_data(self, data) -> None: ...
    def handle_charref(self, name) -> None: ...
    def handle_entityref(self, name) -> None: ...
    def handle_comment(self, data) -> None: ...
    def handle_decl(self, data) -> None: ...
    def unknown_decl(self, data) -> None: ...
    def handle_pi(self, data) -> None: ...

def htmlparser_trace(data) -> None: ...
def rword(length: int = 5): ...
def rsentence(length: int = 4): ...
def rdoc(num_elements: int = 1000): ...
def benchmark_parsers(num_elements: int = 100000) -> None: ...
def profile(num_elements: int = 100000, parser: str = "lxml") -> None: ...



from collections.abc import Callable, Iterator
from contextlib import AbstractContextManager
from typing_extensions import TypeAlias

def known_hosts() -> str: ...
def st_mode_to_int(val: int) -> int: ...

class WTCallbacks:
    def __init__(self) -> None: ...
    def file_cb(self, pathname: str) -> None: ...
    def dir_cb(self, pathname: str) -> None: ...
    def unk_cb(self, pathname: str) -> None: ...
    @property
    def flist(self) -> list[str]: ...
    @flist.setter
    def flist(self, val: list[str]) -> None: ...
    @property
    def dlist(self) -> list[str]: ...
    @dlist.setter
    def dlist(self, val: list[str]) -> None: ...
    @property
    def ulist(self) -> list[str]: ...
    @ulist.setter
    def ulist(self, val: list[str]) -> None: ...

def path_advance(thepath: str, sep: str = ...) -> Iterator[str]: ...
def path_retreat(thepath: str, sep: str = ...) -> Iterator[str]: ...
def reparent(newparent: str, oldpath: str) -> str: ...

_PathCallback: TypeAlias = Callable[[str], object]

def walktree(
    localpath: str, fcallback: _PathCallback, dcallback: _PathCallback, ucallback: _PathCallback, recurse: bool = True
) -> None: ...
def cd(localpath: str | None = None) -> AbstractContextManager[None]: ...



from typing import ClassVar

from .core import UnitedStates

class Nevada(UnitedStates):
    include_thanksgiving_friday: ClassVar[bool]
    thanksgiving_friday_label: ClassVar[str]
    include_columbus_day: ClassVar[bool]
    def get_variable_days(self, year): ...



from collections.abc import Callable

from consolemenu.console_menu import ConsoleMenu
from consolemenu.items import MenuItem as MenuItem

class SubmenuItem(MenuItem):
    submenu: ConsoleMenu
    def __init__(
        self,
        text: str | Callable[[], str],
        submenu: ConsoleMenu,
        menu: ConsoleMenu | None = None,
        should_exit: bool = False,
        menu_char: str | None = None,
    ) -> None: ...
    menu: ConsoleMenu
    def set_menu(self, menu: ConsoleMenu) -> None: ...
    def set_up(self) -> None: ...
    def action(self) -> None: ...
    def clean_up(self) -> None: ...
    def get_return(self) -> object: ...
    def get_submenu(self) -> ConsoleMenu: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def connected_components(G) -> Generator[Incomplete, None, None]: ...
@_dispatch
def number_connected_components(G): ...
@_dispatch
def is_connected(G): ...
@_dispatch
def node_connected_component(G, n): ...



from typing import Final

XK_Greek_ALPHAaccent: Final = 0x7A1
XK_Greek_EPSILONaccent: Final = 0x7A2
XK_Greek_ETAaccent: Final = 0x7A3
XK_Greek_IOTAaccent: Final = 0x7A4
XK_Greek_IOTAdiaeresis: Final = 0x7A5
XK_Greek_OMICRONaccent: Final = 0x7A7
XK_Greek_UPSILONaccent: Final = 0x7A8
XK_Greek_UPSILONdieresis: Final = 0x7A9
XK_Greek_OMEGAaccent: Final = 0x7AB
XK_Greek_accentdieresis: Final = 0x7AE
XK_Greek_horizbar: Final = 0x7AF
XK_Greek_alphaaccent: Final = 0x7B1
XK_Greek_epsilonaccent: Final = 0x7B2
XK_Greek_etaaccent: Final = 0x7B3
XK_Greek_iotaaccent: Final = 0x7B4
XK_Greek_iotadieresis: Final = 0x7B5
XK_Greek_iotaaccentdieresis: Final = 0x7B6
XK_Greek_omicronaccent: Final = 0x7B7
XK_Greek_upsilonaccent: Final = 0x7B8
XK_Greek_upsilondieresis: Final = 0x7B9
XK_Greek_upsilonaccentdieresis: Final = 0x7BA
XK_Greek_omegaaccent: Final = 0x7BB
XK_Greek_ALPHA: Final = 0x7C1
XK_Greek_BETA: Final = 0x7C2
XK_Greek_GAMMA: Final = 0x7C3
XK_Greek_DELTA: Final = 0x7C4
XK_Greek_EPSILON: Final = 0x7C5
XK_Greek_ZETA: Final = 0x7C6
XK_Greek_ETA: Final = 0x7C7
XK_Greek_THETA: Final = 0x7C8
XK_Greek_IOTA: Final = 0x7C9
XK_Greek_KAPPA: Final = 0x7CA
XK_Greek_LAMDA: Final = 0x7CB
XK_Greek_LAMBDA: Final = 0x7CB
XK_Greek_MU: Final = 0x7CC
XK_Greek_NU: Final = 0x7CD
XK_Greek_XI: Final = 0x7CE
XK_Greek_OMICRON: Final = 0x7CF
XK_Greek_PI: Final = 0x7D0
XK_Greek_RHO: Final = 0x7D1
XK_Greek_SIGMA: Final = 0x7D2
XK_Greek_TAU: Final = 0x7D4
XK_Greek_UPSILON: Final = 0x7D5
XK_Greek_PHI: Final = 0x7D6
XK_Greek_CHI: Final = 0x7D7
XK_Greek_PSI: Final = 0x7D8
XK_Greek_OMEGA: Final = 0x7D9
XK_Greek_alpha: Final = 0x7E1
XK_Greek_beta: Final = 0x7E2
XK_Greek_gamma: Final = 0x7E3
XK_Greek_delta: Final = 0x7E4
XK_Greek_epsilon: Final = 0x7E5
XK_Greek_zeta: Final = 0x7E6
XK_Greek_eta: Final = 0x7E7
XK_Greek_theta: Final = 0x7E8
XK_Greek_iota: Final = 0x7E9
XK_Greek_kappa: Final = 0x7EA
XK_Greek_lamda: Final = 0x7EB
XK_Greek_lambda: Final = 0x7EB
XK_Greek_mu: Final = 0x7EC
XK_Greek_nu: Final = 0x7ED
XK_Greek_xi: Final = 0x7EE
XK_Greek_omicron: Final = 0x7EF
XK_Greek_pi: Final = 0x7F0
XK_Greek_rho: Final = 0x7F1
XK_Greek_sigma: Final = 0x7F2
XK_Greek_finalsmallsigma: Final = 0x7F3
XK_Greek_tau: Final = 0x7F4
XK_Greek_upsilon: Final = 0x7F5
XK_Greek_phi: Final = 0x7F6
XK_Greek_chi: Final = 0x7F7
XK_Greek_psi: Final = 0x7F8
XK_Greek_omega: Final = 0x7F9
XK_Greek_switch: Final = 0xFF7E



import sys
from collections.abc import Callable, Iterable, Iterator, Mapping
from types import TracebackType
from typing import Any, Generic, Literal, TypeVar
from typing_extensions import Self

if sys.version_info >= (3, 9):
    from types import GenericAlias

__all__ = ["Pool", "ThreadPool"]

_S = TypeVar("_S")
_T = TypeVar("_T")

class ApplyResult(Generic[_T]):
    def __init__(
        self, pool: Pool, callback: Callable[[_T], object] | None, error_callback: Callable[[BaseException], object] | None
    ) -> None: ...
    def get(self, timeout: float | None = None) -> _T: ...
    def wait(self, timeout: float | None = None) -> None: ...
    def ready(self) -> bool: ...
    def successful(self) -> bool: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

# alias created during issue #17805
AsyncResult = ApplyResult

class MapResult(ApplyResult[list[_T]]):
    def __init__(
        self,
        pool: Pool,
        chunksize: int,
        length: int,
        callback: Callable[[list[_T]], object] | None,
        error_callback: Callable[[BaseException], object] | None,
    ) -> None: ...

class IMapIterator(Iterator[_T]):
    def __init__(self, pool: Pool) -> None: ...
    def __iter__(self) -> Self: ...
    def next(self, timeout: float | None = None) -> _T: ...
    def __next__(self, timeout: float | None = None) -> _T: ...

class IMapUnorderedIterator(IMapIterator[_T]): ...

class Pool:
    def __init__(
        self,
        processes: int | None = None,
        initializer: Callable[..., object] | None = None,
        initargs: Iterable[Any] = (),
        maxtasksperchild: int | None = None,
        context: Any | None = None,
    ) -> None: ...
    def apply(self, func: Callable[..., _T], args: Iterable[Any] = (), kwds: Mapping[str, Any] = {}) -> _T: ...
    def apply_async(
        self,
        func: Callable[..., _T],
        args: Iterable[Any] = (),
        kwds: Mapping[str, Any] = {},
        callback: Callable[[_T], object] | None = None,
        error_callback: Callable[[BaseException], object] | None = None,
    ) -> AsyncResult[_T]: ...
    def map(self, func: Callable[[_S], _T], iterable: Iterable[_S], chunksize: int | None = None) -> list[_T]: ...
    def map_async(
        self,
        func: Callable[[_S], _T],
        iterable: Iterable[_S],
        chunksize: int | None = None,
        callback: Callable[[list[_T]], object] | None = None,
        error_callback: Callable[[BaseException], object] | None = None,
    ) -> MapResult[_T]: ...
    def imap(self, func: Callable[[_S], _T], iterable: Iterable[_S], chunksize: int | None = 1) -> IMapIterator[_T]: ...
    def imap_unordered(self, func: Callable[[_S], _T], iterable: Iterable[_S], chunksize: int | None = 1) -> IMapIterator[_T]: ...
    def starmap(self, func: Callable[..., _T], iterable: Iterable[Iterable[Any]], chunksize: int | None = None) -> list[_T]: ...
    def starmap_async(
        self,
        func: Callable[..., _T],
        iterable: Iterable[Iterable[Any]],
        chunksize: int | None = None,
        callback: Callable[[list[_T]], object] | None = None,
        error_callback: Callable[[BaseException], object] | None = None,
    ) -> AsyncResult[list[_T]]: ...
    def close(self) -> None: ...
    def terminate(self) -> None: ...
    def join(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def __del__(self) -> None: ...

class ThreadPool(Pool):
    def __init__(
        self, processes: int | None = None, initializer: Callable[..., object] | None = None, initargs: Iterable[Any] = ()
    ) -> None: ...

# undocumented
INIT: Literal["INIT"]
RUN: Literal["RUN"]
CLOSE: Literal["CLOSE"]
TERMINATE: Literal["TERMINATE"]



from .china import China as China
from .hong_kong import HongKong as HongKong, HongKongBank as HongKongBank
from .israel import Israel as Israel
from .japan import Japan as Japan, JapanBank as JapanBank
from .kazakhstan import Kazakhstan as Kazakhstan
from .malaysia import Malaysia as Malaysia
from .philippines import Philippines as Philippines
from .qatar import Qatar as Qatar
from .singapore import Singapore as Singapore
from .south_korea import SouthKorea as SouthKorea
from .taiwan import Taiwan as Taiwan



from _typeshed import Incomplete

from networkx.algorithms.flow import edmonds_karp
from networkx.utils.backends import _dispatch

__all__ = ["minimum_st_node_cut", "minimum_node_cut", "minimum_st_edge_cut", "minimum_edge_cut"]

default_flow_func = edmonds_karp

@_dispatch
def minimum_st_edge_cut(
    G, s, t, flow_func: Incomplete | None = None, auxiliary: Incomplete | None = None, residual: Incomplete | None = None
): ...
@_dispatch
def minimum_st_node_cut(
    G, s, t, flow_func: Incomplete | None = None, auxiliary: Incomplete | None = None, residual: Incomplete | None = None
): ...
@_dispatch
def minimum_node_cut(G, s: Incomplete | None = None, t: Incomplete | None = None, flow_func: Incomplete | None = None): ...
@_dispatch
def minimum_edge_cut(G, s: Incomplete | None = None, t: Incomplete | None = None, flow_func: Incomplete | None = None): ...



from pyasn1.type import namedtype, univ

RSA_ENCRYPTION_ASN1_OID: str

class RsaAlgorithmIdentifier(univ.Sequence):
    componentType: namedtype.NamedTypes

class PKCS8PrivateKey(univ.Sequence):
    componentType: namedtype.NamedTypes

class PublicKeyInfo(univ.Sequence):
    componentType: namedtype.NamedTypes

def rsa_private_key_pkcs8_to_pkcs1(pkcs8_key) -> bytes: ...
def rsa_private_key_pkcs1_to_pkcs8(pkcs1_key) -> bytes: ...
def rsa_public_key_pkcs1_to_pkcs8(pkcs1_key) -> bytes: ...
def rsa_public_key_pkcs8_to_pkcs1(pkcs8_key) -> bytes: ...



from _typeshed import Incomplete
from typing import TextIO

from antlr4.atn.LexerATNSimulator import LexerATNSimulator as LexerATNSimulator
from antlr4.CommonTokenFactory import CommonTokenFactory as CommonTokenFactory
from antlr4.error.Errors import (
    IllegalStateException as IllegalStateException,
    LexerNoViableAltException as LexerNoViableAltException,
    RecognitionException as RecognitionException,
)
from antlr4.InputStream import InputStream as InputStream
from antlr4.Recognizer import Recognizer as Recognizer
from antlr4.Token import Token as Token

class TokenSource: ...

class Lexer(Recognizer, TokenSource):
    DEFAULT_MODE: int
    MORE: int
    SKIP: int
    DEFAULT_TOKEN_CHANNEL: Incomplete
    HIDDEN: Incomplete
    MIN_CHAR_VALUE: int
    MAX_CHAR_VALUE: int
    def __init__(self, input: InputStream, output: TextIO = ...) -> None: ...
    def reset(self) -> None: ...
    def nextToken(self): ...
    def skip(self) -> None: ...
    def more(self) -> None: ...
    def mode(self, m: int): ...
    def pushMode(self, m: int): ...
    def popMode(self): ...
    @property
    def inputStream(self): ...
    @inputStream.setter
    def inputStream(self, input: InputStream): ...
    @property
    def sourceName(self): ...
    def emitToken(self, token: Token): ...
    def emit(self): ...
    def emitEOF(self): ...
    @property
    def type(self): ...
    @type.setter
    def type(self, type: int): ...
    @property
    def line(self): ...
    @line.setter
    def line(self, line: int): ...
    @property
    def column(self): ...
    @column.setter
    def column(self, column: int): ...
    def getCharIndex(self): ...
    @property
    def text(self): ...
    @text.setter
    def text(self, txt: str): ...
    def getAllTokens(self): ...
    def notifyListeners(self, e: LexerNoViableAltException): ...
    def getErrorDisplay(self, s: str): ...
    def getErrorDisplayForChar(self, c: str): ...
    def getCharErrorDisplay(self, c: str): ...
    def recover(self, re: RecognitionException): ...



from _typeshed import SupportsItems, SupportsRead
from _typeshed.wsgi import StartResponse, WSGIApplication, WSGIEnvironment
from collections.abc import Iterable, Iterator, Sequence
from datetime import timedelta
from typing import IO, Any, Literal, Protocol, TypedDict
from typing_extensions import TypeAlias

from webob.byterange import ContentRange
from webob.cachecontrol import _ResponseCacheControl
from webob.cookies import _SameSitePolicy
from webob.descriptors import (
    _AsymmetricProperty,
    _AsymmetricPropertyWithDelete,
    _authorization,
    _ContentRangeParams,
    _DateProperty,
    _ListProperty,
)
from webob.headers import ResponseHeaders
from webob.request import Request

class _ResponseCacheExpires(Protocol):
    def __call__(
        self,
        seconds: int | timedelta = 0,
        *,
        public: bool = ...,
        private: Literal[True] | str = ...,
        no_cache: Literal[True] | str = ...,
        no_store: bool = ...,
        no_transform: bool = ...,
        must_revalidate: bool = ...,
        proxy_revalidate: bool = ...,
        max_age: int = ...,
        s_maxage: int = ...,
        s_max_age: int = ...,
        stale_while_revalidate: int = ...,
        stale_if_error: int = ...,
    ) -> None: ...

class _ResponseCacheControlDict(TypedDict, total=False):
    public: bool
    private: Literal[True] | str
    no_cache: Literal[True] | str
    no_store: bool
    no_transform: bool
    must_revalidate: bool
    proxy_revalidate: bool
    max_age: int
    s_maxage: int
    s_max_age: int
    stale_while_revalidate: int
    stale_if_error: int

_HTTPHeader: TypeAlias = tuple[str, str]

class Response:
    default_content_type: str
    default_charset: str
    unicode_errors: str
    default_conditional_response: bool
    default_body_encoding: str
    request: Request | None
    environ: WSGIEnvironment | None
    status: str
    conditional_response: bool

    def __init__(
        self,
        body: bytes | str | None = None,
        status: str | None = None,
        headerlist: list[_HTTPHeader] | None = None,
        app_iter: Iterator[bytes] | None = None,
        content_type: str | None = None,
        conditional_response: bool | None = None,
        charset: str = ...,
        **kw: Any,
    ) -> None: ...
    @classmethod
    def from_file(cls, fp: IO[str]) -> Response: ...
    def copy(self) -> Response: ...
    status_code: int
    status_int: int
    headerlist: _AsymmetricPropertyWithDelete[list[_HTTPHeader], Iterable[_HTTPHeader] | SupportsItems[str, str]]
    headers: _AsymmetricProperty[ResponseHeaders, SupportsItems[str, str] | Iterable[tuple[str, str]]]
    body: bytes
    json: Any
    json_body: Any
    @property
    def has_body(self) -> bool: ...
    text: str
    unicode_body: str  # deprecated
    ubody: str  # deprecated
    body_file: _AsymmetricPropertyWithDelete[ResponseBodyFile, SupportsRead[bytes]]
    content_length: int | None
    def write(self, text: str | bytes) -> None: ...
    app_iter: Iterator[bytes]
    allow: _ListProperty
    vary: _ListProperty
    content_encoding: str | None
    content_language: _ListProperty
    content_location: str | None
    content_md5: str | None
    content_disposition: str | None
    accept_ranges: str | None
    content_range: _AsymmetricPropertyWithDelete[ContentRange | None, _ContentRangeParams]
    date: _DateProperty
    expires: _DateProperty
    last_modified: _DateProperty
    etag: _AsymmetricPropertyWithDelete[str | None, tuple[str, bool] | str | None]
    @property
    def etag_strong(self) -> str | None: ...
    location: str | None
    pragma: str | None
    age: int | None
    retry_after: _DateProperty
    server: str | None
    www_authenticate: _AsymmetricPropertyWithDelete[
        _authorization | None, tuple[str, str | dict[str, str]] | list[Any] | str | None
    ]
    charset: str | None
    content_type: str | None
    content_type_params: _AsymmetricPropertyWithDelete[dict[str, str], SupportsItems[str, str] | None]
    def set_cookie(
        self,
        name: str,
        value: str | None = "",
        max_age: int | timedelta | None = None,
        path: str = "/",
        domain: str | None = None,
        secure: bool = False,
        httponly: bool = False,
        comment: str | None = None,
        overwrite: bool = False,
        samesite: _SameSitePolicy | None = None,
    ) -> None: ...
    def delete_cookie(self, name: str, path: str = "/", domain: str | None = None) -> None: ...
    def unset_cookie(self, name: str, strict: bool = True) -> None: ...
    def merge_cookies(self, resp: Response | WSGIApplication) -> None: ...
    cache_control: _AsymmetricProperty[_ResponseCacheControl, _ResponseCacheControl | _ResponseCacheControlDict | str | None]
    cache_expires: _AsymmetricProperty[_ResponseCacheExpires, timedelta | int | bool | None]
    def encode_content(self, encoding: Literal["gzip", "identity"] = "gzip", lazy: bool = False) -> None: ...
    def decode_content(self) -> None: ...
    def md5_etag(self, body: bytes | None = None, set_content_md5: bool = False) -> None: ...
    def __call__(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...
    def conditional_response_app(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...
    def app_iter_range(self, start: int, stop: int | None) -> AppIterRange: ...
    def __str__(self, skip_body: bool = False) -> str: ...

class ResponseBodyFile:
    mode: Literal["wb"]
    closed: Literal[False]
    response: Response
    def __init__(self, response: Response) -> None: ...
    @property
    def encoding(self) -> str | None: ...
    def write(self, text: str | bytes) -> int: ...
    def writelines(self, seq: Sequence[str | bytes]) -> int: ...
    def flush(self) -> None: ...
    def tell(self) -> int: ...

class AppIterRange:
    app_iter: Iterator[bytes]
    start: int
    stop: int | None
    def __init__(self, app_iter: Iterator[bytes], start: int, stop: int | None) -> None: ...
    def __iter__(self) -> Iterator[bytes]: ...
    def next(self) -> bytes: ...
    __next__ = next
    def close(self) -> None: ...

class EmptyResponse:
    def __init__(self, app_iter: Iterator[bytes] | None = None) -> None: ...
    def __iter__(self) -> Iterator[bytes]: ...
    def __len__(self) -> int: ...
    def next(self) -> bytes: ...
    __next__ = next



class ModulusPack:
    pack: dict[int, list[tuple[int, int]]]
    discarded: list[tuple[int, str]]
    def __init__(self) -> None: ...
    def read_file(self, filename: str) -> None: ...
    def get_modulus(self, min: int, prefer: int, max: int) -> tuple[int, int]: ...



from networkx.utils.backends import _dispatch

@_dispatch
def degree_centrality(G, nodes): ...
@_dispatch
def betweenness_centrality(G, nodes): ...
@_dispatch
def closeness_centrality(G, nodes, normalized: bool = True): ...



from _typeshed import Incomplete
from typing import Any

PARSER_HARDCODED_TOKENS: Any
PARSER_KNOWN_TOKENS: Any
ALWAYS_KEEP_TOKENS: list[str]
KNOWN_WORD_TOKENS: Any
PARENTHESES_PATTERN: Any
NUMERAL_PATTERN: Any
KEEP_TOKEN_PATTERN: Any

class UnknownTokenError(Exception): ...

class Dictionary:
    info: Any
    def __init__(self, locale_info, settings: Incomplete | None = None) -> None: ...
    def __contains__(self, key): ...
    def __getitem__(self, key): ...
    def __iter__(self) -> Any: ...
    def are_tokens_valid(self, tokens): ...
    def split(self, string, keep_formatting: bool = False): ...

class NormalizedDictionary(Dictionary):
    def __init__(self, locale_info, settings: Incomplete | None = None) -> None: ...



from _typeshed import Incomplete

LIST_OF_FILE_NAMES: Incomplete

def get_extension(srcfilename, modname, sources=(), **kwds): ...
def compile(tmpdir, ext, compiler_verbose: int = 0, debug: Incomplete | None = None): ...
def maybe_relative_path(path): ...

int_or_long = int

def flatten(x): ...



def lookup(code: str, _cache: dict[str, str] = {}) -> str: ...

CLASS_SUCCESSFUL_COMPLETION: str
CLASS_WARNING: str
CLASS_NO_DATA: str
CLASS_SQL_STATEMENT_NOT_YET_COMPLETE: str
CLASS_CONNECTION_EXCEPTION: str
CLASS_TRIGGERED_ACTION_EXCEPTION: str
CLASS_FEATURE_NOT_SUPPORTED: str
CLASS_INVALID_TRANSACTION_INITIATION: str
CLASS_LOCATOR_EXCEPTION: str
CLASS_INVALID_GRANTOR: str
CLASS_INVALID_ROLE_SPECIFICATION: str
CLASS_DIAGNOSTICS_EXCEPTION: str
CLASS_CASE_NOT_FOUND: str
CLASS_CARDINALITY_VIOLATION: str
CLASS_DATA_EXCEPTION: str
CLASS_INTEGRITY_CONSTRAINT_VIOLATION: str
CLASS_INVALID_CURSOR_STATE: str
CLASS_INVALID_TRANSACTION_STATE: str
CLASS_INVALID_SQL_STATEMENT_NAME: str
CLASS_TRIGGERED_DATA_CHANGE_VIOLATION: str
CLASS_INVALID_AUTHORIZATION_SPECIFICATION: str
CLASS_DEPENDENT_PRIVILEGE_DESCRIPTORS_STILL_EXIST: str
CLASS_INVALID_TRANSACTION_TERMINATION: str
CLASS_SQL_ROUTINE_EXCEPTION: str
CLASS_INVALID_CURSOR_NAME: str
CLASS_EXTERNAL_ROUTINE_EXCEPTION: str
CLASS_EXTERNAL_ROUTINE_INVOCATION_EXCEPTION: str
CLASS_SAVEPOINT_EXCEPTION: str
CLASS_INVALID_CATALOG_NAME: str
CLASS_INVALID_SCHEMA_NAME: str
CLASS_TRANSACTION_ROLLBACK: str
CLASS_SYNTAX_ERROR_OR_ACCESS_RULE_VIOLATION: str
CLASS_WITH_CHECK_OPTION_VIOLATION: str
CLASS_INSUFFICIENT_RESOURCES: str
CLASS_PROGRAM_LIMIT_EXCEEDED: str
CLASS_OBJECT_NOT_IN_PREREQUISITE_STATE: str
CLASS_OPERATOR_INTERVENTION: str
CLASS_SYSTEM_ERROR: str
CLASS_SNAPSHOT_FAILURE: str
CLASS_CONFIGURATION_FILE_ERROR: str
CLASS_FOREIGN_DATA_WRAPPER_ERROR: str
CLASS_PL_PGSQL_ERROR: str
CLASS_INTERNAL_ERROR: str
SUCCESSFUL_COMPLETION: str
WARNING: str
NULL_VALUE_ELIMINATED_IN_SET_FUNCTION: str
STRING_DATA_RIGHT_TRUNCATION_: str
PRIVILEGE_NOT_REVOKED: str
PRIVILEGE_NOT_GRANTED: str
IMPLICIT_ZERO_BIT_PADDING: str
DYNAMIC_RESULT_SETS_RETURNED: str
DEPRECATED_FEATURE: str
NO_DATA: str
NO_ADDITIONAL_DYNAMIC_RESULT_SETS_RETURNED: str
SQL_STATEMENT_NOT_YET_COMPLETE: str
CONNECTION_EXCEPTION: str
SQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION: str
CONNECTION_DOES_NOT_EXIST: str
SQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION: str
CONNECTION_FAILURE: str
TRANSACTION_RESOLUTION_UNKNOWN: str
PROTOCOL_VIOLATION: str
TRIGGERED_ACTION_EXCEPTION: str
FEATURE_NOT_SUPPORTED: str
INVALID_TRANSACTION_INITIATION: str
LOCATOR_EXCEPTION: str
INVALID_LOCATOR_SPECIFICATION: str
INVALID_GRANTOR: str
INVALID_GRANT_OPERATION: str
INVALID_ROLE_SPECIFICATION: str
DIAGNOSTICS_EXCEPTION: str
STACKED_DIAGNOSTICS_ACCESSED_WITHOUT_ACTIVE_HANDLER: str
CASE_NOT_FOUND: str
CARDINALITY_VIOLATION: str
DATA_EXCEPTION: str
STRING_DATA_RIGHT_TRUNCATION: str
NULL_VALUE_NO_INDICATOR_PARAMETER: str
NUMERIC_VALUE_OUT_OF_RANGE: str
NULL_VALUE_NOT_ALLOWED_: str
ERROR_IN_ASSIGNMENT: str
INVALID_DATETIME_FORMAT: str
DATETIME_FIELD_OVERFLOW: str
INVALID_TIME_ZONE_DISPLACEMENT_VALUE: str
ESCAPE_CHARACTER_CONFLICT: str
INVALID_USE_OF_ESCAPE_CHARACTER: str
INVALID_ESCAPE_OCTET: str
ZERO_LENGTH_CHARACTER_STRING: str
MOST_SPECIFIC_TYPE_MISMATCH: str
SEQUENCE_GENERATOR_LIMIT_EXCEEDED: str
NOT_AN_XML_DOCUMENT: str
INVALID_XML_DOCUMENT: str
INVALID_XML_CONTENT: str
INVALID_XML_COMMENT: str
INVALID_XML_PROCESSING_INSTRUCTION: str
INVALID_INDICATOR_PARAMETER_VALUE: str
SUBSTRING_ERROR: str
DIVISION_BY_ZERO: str
INVALID_PRECEDING_OR_FOLLOWING_SIZE: str
INVALID_ARGUMENT_FOR_NTILE_FUNCTION: str
INTERVAL_FIELD_OVERFLOW: str
INVALID_ARGUMENT_FOR_NTH_VALUE_FUNCTION: str
INVALID_CHARACTER_VALUE_FOR_CAST: str
INVALID_ESCAPE_CHARACTER: str
INVALID_REGULAR_EXPRESSION: str
INVALID_ARGUMENT_FOR_LOGARITHM: str
INVALID_ARGUMENT_FOR_POWER_FUNCTION: str
INVALID_ARGUMENT_FOR_WIDTH_BUCKET_FUNCTION: str
INVALID_ROW_COUNT_IN_LIMIT_CLAUSE: str
INVALID_ROW_COUNT_IN_RESULT_OFFSET_CLAUSE: str
INVALID_LIMIT_VALUE: str
CHARACTER_NOT_IN_REPERTOIRE: str
INDICATOR_OVERFLOW: str
INVALID_PARAMETER_VALUE: str
UNTERMINATED_C_STRING: str
INVALID_ESCAPE_SEQUENCE: str
STRING_DATA_LENGTH_MISMATCH: str
TRIM_ERROR: str
ARRAY_SUBSCRIPT_ERROR: str
INVALID_TABLESAMPLE_REPEAT: str
INVALID_TABLESAMPLE_ARGUMENT: str
DUPLICATE_JSON_OBJECT_KEY_VALUE: str
INVALID_ARGUMENT_FOR_SQL_JSON_DATETIME_FUNCTION: str
INVALID_JSON_TEXT: str
INVALID_SQL_JSON_SUBSCRIPT: str
MORE_THAN_ONE_SQL_JSON_ITEM: str
NO_SQL_JSON_ITEM: str
NON_NUMERIC_SQL_JSON_ITEM: str
NON_UNIQUE_KEYS_IN_A_JSON_OBJECT: str
SINGLETON_SQL_JSON_ITEM_REQUIRED: str
SQL_JSON_ARRAY_NOT_FOUND: str
SQL_JSON_MEMBER_NOT_FOUND: str
SQL_JSON_NUMBER_NOT_FOUND: str
SQL_JSON_OBJECT_NOT_FOUND: str
TOO_MANY_JSON_ARRAY_ELEMENTS: str
TOO_MANY_JSON_OBJECT_MEMBERS: str
SQL_JSON_SCALAR_REQUIRED: str
FLOATING_POINT_EXCEPTION: str
INVALID_TEXT_REPRESENTATION: str
INVALID_BINARY_REPRESENTATION: str
BAD_COPY_FILE_FORMAT: str
UNTRANSLATABLE_CHARACTER: str
NONSTANDARD_USE_OF_ESCAPE_CHARACTER: str
INTEGRITY_CONSTRAINT_VIOLATION: str
RESTRICT_VIOLATION: str
NOT_NULL_VIOLATION: str
FOREIGN_KEY_VIOLATION: str
UNIQUE_VIOLATION: str
CHECK_VIOLATION: str
EXCLUSION_VIOLATION: str
INVALID_CURSOR_STATE: str
INVALID_TRANSACTION_STATE: str
ACTIVE_SQL_TRANSACTION: str
BRANCH_TRANSACTION_ALREADY_ACTIVE: str
INAPPROPRIATE_ACCESS_MODE_FOR_BRANCH_TRANSACTION: str
INAPPROPRIATE_ISOLATION_LEVEL_FOR_BRANCH_TRANSACTION: str
NO_ACTIVE_SQL_TRANSACTION_FOR_BRANCH_TRANSACTION: str
READ_ONLY_SQL_TRANSACTION: str
SCHEMA_AND_DATA_STATEMENT_MIXING_NOT_SUPPORTED: str
HELD_CURSOR_REQUIRES_SAME_ISOLATION_LEVEL: str
NO_ACTIVE_SQL_TRANSACTION: str
IN_FAILED_SQL_TRANSACTION: str
IDLE_IN_TRANSACTION_SESSION_TIMEOUT: str
INVALID_SQL_STATEMENT_NAME: str
TRIGGERED_DATA_CHANGE_VIOLATION: str
INVALID_AUTHORIZATION_SPECIFICATION: str
INVALID_PASSWORD: str
DEPENDENT_PRIVILEGE_DESCRIPTORS_STILL_EXIST: str
DEPENDENT_OBJECTS_STILL_EXIST: str
INVALID_TRANSACTION_TERMINATION: str
SQL_ROUTINE_EXCEPTION: str
MODIFYING_SQL_DATA_NOT_PERMITTED_: str
PROHIBITED_SQL_STATEMENT_ATTEMPTED_: str
READING_SQL_DATA_NOT_PERMITTED_: str
FUNCTION_EXECUTED_NO_RETURN_STATEMENT: str
INVALID_CURSOR_NAME: str
EXTERNAL_ROUTINE_EXCEPTION: str
CONTAINING_SQL_NOT_PERMITTED: str
MODIFYING_SQL_DATA_NOT_PERMITTED: str
PROHIBITED_SQL_STATEMENT_ATTEMPTED: str
READING_SQL_DATA_NOT_PERMITTED: str
EXTERNAL_ROUTINE_INVOCATION_EXCEPTION: str
INVALID_SQLSTATE_RETURNED: str
NULL_VALUE_NOT_ALLOWED: str
TRIGGER_PROTOCOL_VIOLATED: str
SRF_PROTOCOL_VIOLATED: str
EVENT_TRIGGER_PROTOCOL_VIOLATED: str
SAVEPOINT_EXCEPTION: str
INVALID_SAVEPOINT_SPECIFICATION: str
INVALID_CATALOG_NAME: str
INVALID_SCHEMA_NAME: str
TRANSACTION_ROLLBACK: str
SERIALIZATION_FAILURE: str
TRANSACTION_INTEGRITY_CONSTRAINT_VIOLATION: str
STATEMENT_COMPLETION_UNKNOWN: str
DEADLOCK_DETECTED: str
SYNTAX_ERROR_OR_ACCESS_RULE_VIOLATION: str
INSUFFICIENT_PRIVILEGE: str
SYNTAX_ERROR: str
INVALID_NAME: str
INVALID_COLUMN_DEFINITION: str
NAME_TOO_LONG: str
DUPLICATE_COLUMN: str
AMBIGUOUS_COLUMN: str
UNDEFINED_COLUMN: str
UNDEFINED_OBJECT: str
DUPLICATE_OBJECT: str
DUPLICATE_ALIAS: str
DUPLICATE_FUNCTION: str
AMBIGUOUS_FUNCTION: str
GROUPING_ERROR: str
DATATYPE_MISMATCH: str
WRONG_OBJECT_TYPE: str
INVALID_FOREIGN_KEY: str
CANNOT_COERCE: str
UNDEFINED_FUNCTION: str
GENERATED_ALWAYS: str
RESERVED_NAME: str
UNDEFINED_TABLE: str
UNDEFINED_PARAMETER: str
DUPLICATE_CURSOR: str
DUPLICATE_DATABASE: str
DUPLICATE_PREPARED_STATEMENT: str
DUPLICATE_SCHEMA: str
DUPLICATE_TABLE: str
AMBIGUOUS_PARAMETER: str
AMBIGUOUS_ALIAS: str
INVALID_COLUMN_REFERENCE: str
INVALID_CURSOR_DEFINITION: str
INVALID_DATABASE_DEFINITION: str
INVALID_FUNCTION_DEFINITION: str
INVALID_PREPARED_STATEMENT_DEFINITION: str
INVALID_SCHEMA_DEFINITION: str
INVALID_TABLE_DEFINITION: str
INVALID_OBJECT_DEFINITION: str
INDETERMINATE_DATATYPE: str
INVALID_RECURSION: str
WINDOWING_ERROR: str
COLLATION_MISMATCH: str
INDETERMINATE_COLLATION: str
WITH_CHECK_OPTION_VIOLATION: str
INSUFFICIENT_RESOURCES: str
DISK_FULL: str
OUT_OF_MEMORY: str
TOO_MANY_CONNECTIONS: str
CONFIGURATION_LIMIT_EXCEEDED: str
PROGRAM_LIMIT_EXCEEDED: str
STATEMENT_TOO_COMPLEX: str
TOO_MANY_COLUMNS: str
TOO_MANY_ARGUMENTS: str
OBJECT_NOT_IN_PREREQUISITE_STATE: str
OBJECT_IN_USE: str
CANT_CHANGE_RUNTIME_PARAM: str
LOCK_NOT_AVAILABLE: str
UNSAFE_NEW_ENUM_VALUE_USAGE: str
OPERATOR_INTERVENTION: str
QUERY_CANCELED: str
ADMIN_SHUTDOWN: str
CRASH_SHUTDOWN: str
CANNOT_CONNECT_NOW: str
DATABASE_DROPPED: str
SYSTEM_ERROR: str
IO_ERROR: str
UNDEFINED_FILE: str
DUPLICATE_FILE: str
SNAPSHOT_TOO_OLD: str
CONFIG_FILE_ERROR: str
LOCK_FILE_EXISTS: str
FDW_ERROR: str
FDW_OUT_OF_MEMORY: str
FDW_DYNAMIC_PARAMETER_VALUE_NEEDED: str
FDW_INVALID_DATA_TYPE: str
FDW_COLUMN_NAME_NOT_FOUND: str
FDW_INVALID_DATA_TYPE_DESCRIPTORS: str
FDW_INVALID_COLUMN_NAME: str
FDW_INVALID_COLUMN_NUMBER: str
FDW_INVALID_USE_OF_NULL_POINTER: str
FDW_INVALID_STRING_FORMAT: str
FDW_INVALID_HANDLE: str
FDW_INVALID_OPTION_INDEX: str
FDW_INVALID_OPTION_NAME: str
FDW_OPTION_NAME_NOT_FOUND: str
FDW_REPLY_HANDLE: str
FDW_UNABLE_TO_CREATE_EXECUTION: str
FDW_UNABLE_TO_CREATE_REPLY: str
FDW_UNABLE_TO_ESTABLISH_CONNECTION: str
FDW_NO_SCHEMAS: str
FDW_SCHEMA_NOT_FOUND: str
FDW_TABLE_NOT_FOUND: str
FDW_FUNCTION_SEQUENCE_ERROR: str
FDW_TOO_MANY_HANDLES: str
FDW_INCONSISTENT_DESCRIPTOR_INFORMATION: str
FDW_INVALID_ATTRIBUTE_VALUE: str
FDW_INVALID_STRING_LENGTH_OR_BUFFER_LENGTH: str
FDW_INVALID_DESCRIPTOR_FIELD_IDENTIFIER: str
PLPGSQL_ERROR: str
RAISE_EXCEPTION: str
NO_DATA_FOUND: str
TOO_MANY_ROWS: str
ASSERT_FAILURE: str
INTERNAL_ERROR: str
DATA_CORRUPTED: str
INDEX_CORRUPTED: str
IDLE_SESSION_TIMEOUT: str
SQL_JSON_ITEM_CANNOT_BE_CAST_TO_TARGET_TYPE: str



from _typeshed import Incomplete

class SubscriptionGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def cancel(self, subscription_id): ...
    def create(self, params: Incomplete | None = None): ...
    def find(self, subscription_id): ...
    def retry_charge(self, subscription_id, amount: Incomplete | None = None, submit_for_settlement: bool = False): ...
    def search(self, *query): ...
    def update(self, subscription_id, params: Incomplete | None = None): ...



from _typeshed import SupportsItems, SupportsKeysAndGetItem
from _typeshed.wsgi import WSGIEnvironment
from cgi import FieldStorage
from collections.abc import Collection, Iterable, Iterator, MutableMapping
from typing import Any, Literal, TypeVar, overload
from typing_extensions import Self

_T = TypeVar("_T")
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")

class MultiDict(MutableMapping[_KT, _VT]):
    @overload
    def __init__(self, m: SupportsItems[_KT, _VT], /, **kwargs: _VT) -> None: ...
    @overload
    def __init__(self, m: Iterable[tuple[_KT, _VT]], /, **kwargs: _VT) -> None: ...
    @overload
    def __init__(self, **kwargs: _VT) -> None: ...
    @classmethod
    def view_list(cls, lst: list[tuple[_KT, _VT]]) -> MultiDict[_KT, _VT]: ...
    @classmethod
    def from_fieldstorage(cls, fs: FieldStorage) -> MultiDict[str, str | FieldStorage]: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    def __setitem__(self, key: _KT, value: _VT) -> None: ...
    def add(self, key: _KT, value: _VT) -> None: ...
    @overload
    def get(self, key: _KT) -> _VT | None: ...
    @overload
    def get(self, key: _KT, default: _T) -> _VT | _T: ...
    def getall(self, key: _KT) -> list[_VT]: ...
    def getone(self, key: _KT) -> _VT: ...
    def mixed(self) -> dict[_KT, _VT | list[_VT]]: ...
    def dict_of_lists(self) -> dict[_KT, list[_VT]]: ...
    def __delitem__(self, key: _KT) -> None: ...
    def __contains__(self, key: object) -> bool: ...
    has_key = __contains__
    def clear(self) -> None: ...
    def copy(self) -> Self: ...
    @overload
    def setdefault(self, key: _KT, default: None = None) -> _VT | None: ...
    @overload
    def setdefault(self, key: _KT, default: _VT) -> _VT: ...
    @overload
    def pop(self, key: _KT) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _T) -> _VT | _T: ...
    def popitem(self) -> tuple[_KT, _VT]: ...
    @overload  # type: ignore[override]
    def update(self, m: Collection[tuple[_KT, _VT]], /, **kwargs: _VT) -> None: ...
    @overload
    def update(self, **kwargs: _VT) -> None: ...
    @overload
    def extend(self, other: SupportsItems[_KT, _VT], **kwargs: _VT) -> None: ...
    @overload
    def extend(self, other: SupportsKeysAndGetItem[_KT, _VT], **kwargs: _VT) -> None: ...
    @overload
    def extend(self, other: Iterable[tuple[_KT, _VT]], **kwargs: _VT) -> None: ...
    @overload
    def extend(self, other: None = None, **kwargs: _VT) -> None: ...
    def __len__(self) -> int: ...
    def keys(self) -> Iterator[_KT]: ...  # type: ignore[override]
    __iter__ = keys
    def values(self) -> Iterator[_VT]: ...  # type: ignore[override]
    def items(self) -> Iterator[tuple[_KT, _VT]]: ...  # type: ignore[override]

class GetDict(MultiDict[str, str]):
    env: WSGIEnvironment
    @overload
    def __init__(self, data: SupportsItems[str, str], env: WSGIEnvironment) -> None: ...
    @overload
    def __init__(self, data: Iterable[tuple[str, str]], env: WSGIEnvironment) -> None: ...
    def on_change(self) -> None: ...

class NestedMultiDict(MultiDict[_KT, _VT]):
    dicts: tuple[MultiDict[_KT, _VT] | NoVars, ...]
    def __init__(self, *dicts: MultiDict[_KT, _VT] | NoVars) -> None: ...
    def __setitem__(self, key: _KT, value: _VT) -> None: ...
    def add(self, key: _KT, value: _VT) -> None: ...
    def __delitem__(self, key: _KT) -> None: ...
    def clear(self) -> None: ...
    def setdefault(self, key: _KT, default: _VT | None = ...) -> Any: ...
    def pop(self, key: _KT, default: Any = ...) -> Any: ...
    def popitem(self) -> tuple[_KT, _VT]: ...
    def update(self, *args: Any, **kwargs: _VT) -> None: ...
    def copy(self) -> MultiDict[_KT, _VT]: ...  # type: ignore[override]

class NoVars:
    reason: str
    def __init__(self, reason: str | None = None) -> None: ...
    @overload
    def get(self, key: str, default: None = None) -> None: ...
    @overload
    def get(self, key: str, default: _T) -> _T: ...
    def getall(self, key: str) -> list[str]: ...
    def mixed(self) -> dict[str, str | list[str]]: ...
    def dict_of_lists(self) -> dict[str, list[str]]: ...
    def __contains__(self, key: object) -> Literal[False]: ...
    has_key = __contains__
    def copy(self) -> Self: ...
    def __len__(self) -> Literal[0]: ...
    def __iter__(self) -> Iterator[str]: ...
    def keys(self) -> Iterator[str]: ...
    def values(self) -> Iterator[str]: ...
    def items(self) -> Iterator[tuple[str, str]]: ...
    def __bool__(self) -> Literal[False]: ...



from _typeshed import Incomplete
from collections.abc import Generator

from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token
from antlr4.tree.Tree import (
    INVALID_INTERVAL as INVALID_INTERVAL,
    ErrorNodeImpl as ErrorNodeImpl,
    ParseTree as ParseTree,
    ParseTreeListener as ParseTreeListener,
    TerminalNode as TerminalNode,
    TerminalNodeImpl as TerminalNodeImpl,
)

class ParserRuleContext(RuleContext):
    children: Incomplete
    start: Incomplete
    stop: Incomplete
    exception: Incomplete
    def __init__(self, parent: ParserRuleContext | None = None, invokingStateNumber: int | None = None) -> None: ...
    parentCtx: Incomplete
    invokingState: Incomplete
    def copyFrom(self, ctx: ParserRuleContext): ...
    def enterRule(self, listener: ParseTreeListener): ...
    def exitRule(self, listener: ParseTreeListener): ...
    def addChild(self, child: ParseTree): ...
    def removeLastChild(self) -> None: ...
    def addTokenNode(self, token: Token): ...
    def addErrorNode(self, badToken: Token): ...
    def getChild(self, i: int, ttype: type | None = None): ...
    def getChildren(self, predicate: Incomplete | None = None) -> Generator[Incomplete, None, None]: ...
    def getToken(self, ttype: int, i: int): ...
    def getTokens(self, ttype: int): ...
    def getTypedRuleContext(self, ctxType: type, i: int): ...
    def getTypedRuleContexts(self, ctxType: type): ...
    def getChildCount(self): ...
    def getSourceInterval(self): ...

class InterpreterRuleContext(ParserRuleContext):
    ruleIndex: Incomplete
    def __init__(self, parent: ParserRuleContext, invokingStateNumber: int, ruleIndex: int) -> None: ...



from _typeshed import Incomplete
from typing import Any

log: Any

def signature_base_string(http_method: str, base_str_uri: str, normalized_encoded_request_parameters: str) -> str: ...
def base_string_uri(uri: str, host: str | None = None) -> str: ...
def collect_parameters(
    uri_query: str = "",
    body: Incomplete | None = None,
    headers: Incomplete | None = None,
    exclude_oauth_signature: bool = True,
    with_realm: bool = False,
): ...
def normalize_parameters(params) -> str: ...
def sign_hmac_sha1_with_client(sig_base_str, client): ...
def verify_hmac_sha1(request, client_secret: Incomplete | None = None, resource_owner_secret: Incomplete | None = None): ...
def sign_hmac_sha1(base_string, client_secret, resource_owner_secret): ...
def sign_hmac_sha256_with_client(sig_base_str, client): ...
def verify_hmac_sha256(request, client_secret: Incomplete | None = None, resource_owner_secret: Incomplete | None = None): ...
def sign_hmac_sha256(base_string, client_secret, resource_owner_secret): ...
def sign_hmac_sha512_with_client(sig_base_str: str, client): ...
def verify_hmac_sha512(request, client_secret: str | None = None, resource_owner_secret: str | None = None): ...
def sign_rsa_sha1_with_client(sig_base_str, client): ...
def verify_rsa_sha1(request, rsa_public_key: str): ...
def sign_rsa_sha1(base_string, rsa_private_key): ...
def sign_rsa_sha256_with_client(sig_base_str: str, client): ...
def verify_rsa_sha256(request, rsa_public_key: str): ...
def sign_rsa_sha512_with_client(sig_base_str: str, client): ...
def verify_rsa_sha512(request, rsa_public_key: str): ...
def sign_plaintext_with_client(_signature_base_string, client): ...
def sign_plaintext(client_secret, resource_owner_secret): ...
def verify_plaintext(request, client_secret: Incomplete | None = None, resource_owner_secret: Incomplete | None = None): ...



from _typeshed import StrOrBytesPath
from typing import Any

from .exceptions import TrashPermissionError as TrashPermissionError

# The list should be list[StrOrBytesPath] but that doesn't work because invariance
def send2trash(paths: list[Any] | StrOrBytesPath) -> None: ...



import sys
from _typeshed import SupportsItems
from collections.abc import Iterable
from typing import Any, Generic, TypeVar, overload

__all__ = ["TopologicalSorter", "CycleError"]

_T = TypeVar("_T")

if sys.version_info >= (3, 11):
    from types import GenericAlias

class TopologicalSorter(Generic[_T]):
    @overload
    def __init__(self, graph: None = None) -> None: ...
    @overload
    def __init__(self, graph: SupportsItems[_T, Iterable[_T]]) -> None: ...
    def add(self, node: _T, *predecessors: _T) -> None: ...
    def prepare(self) -> None: ...
    def is_active(self) -> bool: ...
    def __bool__(self) -> bool: ...
    def done(self, *nodes: _T) -> None: ...
    def get_ready(self) -> tuple[_T, ...]: ...
    def static_order(self) -> Iterable[_T]: ...
    if sys.version_info >= (3, 11):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class CycleError(ValueError): ...



from _typeshed import Incomplete

def temporary_chdir(new_dir) -> None: ...
def get_file_directory(): ...
def get_temp_directory(): ...
def get_themes_directory(theme_name: Incomplete | None = None, png: bool = False): ...
def create_directory(directory): ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class DeleteService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def post_delete(self, delete_predicate_request, **kwargs): ...
    def post_delete_with_http_info(self, delete_predicate_request, **kwargs): ...
    async def post_delete_async(self, delete_predicate_request, **kwargs): ...



import optparse
from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from configparser import RawConfigParser
from typing import Any, ClassVar

from docutils import SettingsSpec
from docutils.parsers import Parser
from docutils.utils import DependencyList

__docformat__: str

def store_multiple(option, opt, value, parser, *args, **kwargs) -> None: ...
def read_config_file(option, opt, value, parser) -> None: ...
def validate_encoding(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
): ...
def validate_encoding_error_handler(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
): ...
def validate_encoding_and_error_handler(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
): ...
def validate_boolean(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> bool: ...
def validate_nonnegative_int(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> int: ...
def validate_threshold(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> int: ...
def validate_colon_separated_string_list(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> list[str]: ...
def validate_comma_separated_list(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> list[str]: ...
def validate_url_trailing_slash(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> str: ...
def validate_dependency_file(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> DependencyList: ...
def validate_strip_class(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
): ...
def validate_smartquotes_locales(
    setting, value, option_parser, config_parser: Incomplete | None = None, config_section: Incomplete | None = None
) -> list[tuple[str, str]]: ...
def make_paths_absolute(pathdict, keys, base_path: Incomplete | None = None) -> None: ...
def make_one_path_absolute(base_path, path) -> str: ...
def filter_settings_spec(settings_spec, *exclude, **replace) -> tuple[Any, ...]: ...

class Values(optparse.Values):
    def update(self, other_dict, option_parser) -> None: ...
    def copy(self) -> Values: ...

class Option(optparse.Option): ...

class OptionParser(optparse.OptionParser, SettingsSpec):
    standard_config_files: ClassVar[list[str]]
    threshold_choices: ClassVar[list[str]]
    thresholds: ClassVar[dict[str, int]]
    booleans: ClassVar[dict[str, bool]]
    default_error_encoding: ClassVar[str]
    default_error_encoding_error_handler: ClassVar[str]
    config_section: ClassVar[str]
    version_template: ClassVar[str]
    def __init__(
        self,
        components: Iterable[type[Parser]] = (),
        defaults: Mapping[str, Any] | None = None,
        read_config_files: bool | None = False,
        *args,
        **kwargs,
    ) -> None: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class ConfigParser(RawConfigParser):
    def __getattr__(self, name: str) -> Incomplete: ...

class ConfigDeprecationWarning(DeprecationWarning): ...

def get_default_settings(*components) -> Values: ...



from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from re import Match, Pattern
from types import ModuleType
from typing import Any
from typing_extensions import TypeAlias

from docutils import nodes
from docutils.utils import Reporter

class Struct:
    def __init__(self, **keywordargs) -> None: ...

_BasicDefinition: TypeAlias = tuple[str, str, str, list[Pattern[str]]]
_DefinitionParts: TypeAlias = tuple[str, str, str, list[Pattern[str] | _BasicDefinition]]
_DefinitionType: TypeAlias = tuple[str, str, str, list[Pattern[str] | _DefinitionParts]]

class Inliner:
    implicit_dispatch: list[tuple[Pattern[str], Callable[[Match[str], int], Sequence[nodes.Node]]]]
    def __init__(self) -> None: ...
    start_string_prefix: str
    end_string_suffix: str
    parts: _DefinitionType
    patterns: Any
    def init_customizations(self, settings: Any) -> None: ...
    reporter: Reporter
    document: nodes.document
    language: ModuleType
    parent: nodes.Element
    def parse(
        self, text: str, lineno: int, memo: Struct, parent: nodes.Element
    ) -> tuple[list[nodes.Node], list[nodes.system_message]]: ...
    non_whitespace_before: str
    non_whitespace_escape_before: str
    non_unescaped_whitespace_escape_before: str
    non_whitespace_after: str
    simplename: str
    uric: str
    uri_end_delim: str
    urilast: str
    uri_end: str
    emailc: str
    email_pattern: str
    def quoted_start(self, match: Match[str]) -> bool: ...
    def inline_obj(
        self,
        match: Match[str],
        lineno: int,
        end_pattern: Pattern[str],
        nodeclass: nodes.TextElement,
        restore_backslashes: bool = False,
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message], str]: ...
    def problematic(self, text: str, rawsource: str, message: nodes.system_message) -> nodes.problematic: ...
    def emphasis(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def strong(self, match: Match[str], lineno: int) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def interpreted_or_phrase_ref(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def phrase_ref(
        self, before: str, after: str, rawsource: str, escaped: str, text: str | None = None
    ) -> tuple[str, list[nodes.Node], str, list[nodes.Node]]: ...
    def adjust_uri(self, uri: str) -> str: ...
    def interpreted(
        self, rawsource: str, text: str, role: str, lineno: int
    ) -> tuple[list[nodes.Node], list[nodes.system_message]]: ...
    def literal(self, match: Match[str], lineno: int) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def inline_internal_target(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def substitution_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def footnote_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def reference(
        self, match: Match[str], lineno: int, anonymous: bool = ...
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def anonymous_reference(
        self, match: Match[str], lineno: int
    ) -> tuple[str, list[nodes.problematic], str, list[nodes.system_message]]: ...
    def standalone_uri(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    def pep_reference(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    rfc_url: str = ...
    def rfc_reference(
        self, match: Match[str], lineno: int
    ) -> list[tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]: ...
    def implicit_inline(self, text: str, lineno: int) -> list[nodes.Text]: ...
    dispatch: dict[str, Callable[[Match[str], int], tuple[str, list[nodes.problematic], str, list[nodes.system_message]]]] = ...

def __getattr__(name: str) -> Incomplete: ...



import datetime
from _typeshed import Incomplete, StrPath, Unused
from collections.abc import Callable, Generator, Iterable, Sequence
from contextlib import _GeneratorContextManager
from io import BytesIO
from pathlib import PurePath
from re import Pattern
from typing import Any, ClassVar, Final, Literal, NamedTuple, overload
from typing_extensions import TypeAlias, deprecated

from fpdf import ViewerPreferences
from PIL import Image

from .annotations import AnnotationDict, PDFEmbeddedFile
from .drawing import DeviceGray, DeviceRGB, DrawingContext, PaintedPath
from .enums import (
    Align,
    AnnotationFlag,
    AnnotationName,
    Corner,
    FileAttachmentAnnotationName,
    MethodReturnValue,
    PageLayout,
    PathPaintRule,
    RenderStyle,
    TableBordersLayout,
    TableCellFillMode,
    TextMarkupType,
    TextMode as TextMode,
    WrapMode as WrapMode,
    XPos as XPos,
    YPos as YPos,
)
from .errors import FPDFException as FPDFException
from .fonts import FontFace
from .graphics_state import GraphicsStateMixin
from .html import HTML2FPDF
from .image_datastructures import (
    ImageCache,
    ImageInfo as ImageInfo,
    RasterImageInfo as RasterImageInfo,
    VectorImageInfo as VectorImageInfo,
    _AlignLiteral,
)
from .output import OutputProducer, PDFPage
from .recorder import FPDFRecorder
from .structure_tree import StructureTreeBuilder
from .syntax import DestinationXYZ
from .table import Table
from .util import _Unit

__all__ = [
    "FPDF",
    "XPos",
    "YPos",
    "get_page_format",
    "ImageInfo",
    "RasterImageInfo",
    "VectorImageInfo",
    "TextMode",
    "TitleStyle",
    "PAGE_FORMATS",
]

_Orientation: TypeAlias = Literal["", "portrait", "p", "P", "landscape", "l", "L"]
_Format: TypeAlias = Literal["", "a3", "A3", "a4", "A4", "a5", "A5", "letter", "Letter", "legal", "Legal"]
_FontStyle: TypeAlias = Literal["", "B", "I"]
_FontStyles: TypeAlias = Literal["", "B", "I", "U", "BU", "UB", "BI", "IB", "IU", "UI", "BIU", "BUI", "IBU", "IUB", "UBI", "UIB"]

FPDF_VERSION: Final[str]
PAGE_FORMATS: dict[_Format, tuple[float, float]]

class TitleStyle(FontFace):
    t_margin: int | None
    l_margin: int | None
    b_margin: int | None

    def __init__(
        self,
        font_family: str | None = None,
        font_style: str | None = None,
        font_size_pt: int | None = None,
        color: int | tuple[int, int, int] | None = None,
        underline: bool = False,
        t_margin: int | None = None,
        l_margin: int | None = None,
        b_margin: int | None = None,
    ) -> None: ...

class ToCPlaceholder(NamedTuple):
    render_function: Callable[[FPDF, Any], object]
    start_page: int
    y: int
    pages: int = ...

def get_page_format(format: _Format | tuple[float, float], k: float | None = None) -> tuple[float, float]: ...

# TODO: TypedDicts
_Font: TypeAlias = dict[str, Any]

class FPDF(GraphicsStateMixin):
    MARKDOWN_BOLD_MARKER: ClassVar[str]
    MARKDOWN_ITALICS_MARKER: ClassVar[str]
    MARKDOWN_UNDERLINE_MARKER: ClassVar[str]
    MARKDOWN_LINK_REGEX: ClassVar[Pattern[str]]
    MARKDOWN_LINK_COLOR: ClassVar[Incomplete | None]
    MARKDOWN_LINK_UNDERLINE: ClassVar[bool]

    HTML2FPDF_CLASS: ClassVar[type[HTML2FPDF]]

    page: int
    pages: dict[int, PDFPage]
    fonts: dict[str, _Font]
    links: dict[int, DestinationXYZ]
    embedded_files: list[PDFEmbeddedFile]
    image_cache: ImageCache

    in_footer: bool
    str_alias_nb_pages: str

    xmp_metadata: str | None
    page_duration: int
    page_transition: Incomplete | None
    allow_images_transparency: bool
    oversized_images: Incomplete | None
    oversized_images_ratio: float
    struct_builder: StructureTreeBuilder
    section_title_styles: dict[int, Incomplete]

    core_fonts: dict[str, str]
    core_fonts_encoding: str
    font_aliases: dict[str, str]
    k: float

    page_background: Incomplete | None

    dw_pt: float
    dh_pt: float
    def_orientation: Literal["P", "L"]
    x: float
    y: float
    l_margin: float
    t_margin: float
    c_margin: float
    viewer_preferences: ViewerPreferences | None
    compress: bool
    pdf_version: str
    creation_date: datetime.datetime

    buffer: bytearray | None

    # Set during call to _set_orientation(), called from __init__().
    cur_orientation: Literal["P", "L"]
    w_pt: float
    h_pt: float
    w: float
    h: float

    def __init__(
        self,
        orientation: _Orientation = "portrait",
        unit: _Unit | float = "mm",
        format: _Format | tuple[float, float] = "A4",
        font_cache_dir: Literal["DEPRECATED"] = "DEPRECATED",
    ) -> None: ...
    # The following definition crashes stubtest 1.1.1.
    # def set_encryption(
    #     self,
    #     owner_password: str,
    #     user_password: str | None = None,
    #     encryption_method: EncryptionMethod | str = ...,
    #     permissions: AccessPermission = ...,
    #     encrypt_metadata: bool = False,
    # ) -> None: ...
    # args and kwargs are passed to HTML2FPDF_CLASS constructor.
    def write_html(self, text: str, *args: Any, **kwargs: Any) -> None: ...
    @property
    def is_ttf_font(self) -> bool: ...
    @property
    def page_mode(self): ...
    @property
    def epw(self) -> float: ...
    @property
    def eph(self) -> float: ...
    @property
    def pages_count(self) -> int: ...
    def set_margin(self, margin: float) -> None: ...
    def set_margins(self, left: float, top: float, right: float = -1) -> None: ...
    def set_left_margin(self, margin: float) -> None: ...
    def set_top_margin(self, margin: float) -> None: ...
    r_margin: float
    def set_right_margin(self, margin: float) -> None: ...
    auto_page_break: bool
    b_margin: float
    page_break_trigger: float
    def set_auto_page_break(self, auto: bool, margin: float = 0) -> None: ...
    @property
    def default_page_dimensions(self) -> tuple[float, float]: ...
    zoom_mode: Literal["fullpage", "fullwidth", "real", "default"] | float
    page_layout: PageLayout | None
    def set_display_mode(
        self,
        zoom: Literal["fullpage", "fullwidth", "real", "default"] | float,
        layout: Literal["single", "continuous", "two", "default"] = "continuous",
    ) -> None: ...
    def set_text_shaping(
        self,
        use_shaping_engine: bool = True,
        features: dict[str, bool] | None = None,
        direction: Literal["ltr", "rtl"] | None = None,
        script: str | None = None,
        language: str | None = None,
    ) -> None: ...
    def set_compression(self, compress: bool) -> None: ...
    title: str
    def set_title(self, title: str) -> None: ...
    lang: str
    def set_lang(self, lang: str) -> None: ...
    subject: str
    def set_subject(self, subject: str) -> None: ...
    author: str
    def set_author(self, author: str) -> None: ...
    keywords: str
    def set_keywords(self, keywords: str) -> None: ...
    creator: str
    def set_creator(self, creator: str) -> None: ...
    producer: str
    def set_producer(self, producer: str) -> None: ...
    def set_creation_date(self, date: datetime.datetime) -> None: ...
    def set_xmp_metadata(self, xmp_metadata: str) -> None: ...
    def set_doc_option(self, opt: str, value: str) -> None: ...
    def set_image_filter(self, image_filter: str) -> None: ...
    def alias_nb_pages(self, alias: str = "{nb}") -> None: ...
    def add_page(
        self,
        orientation: _Orientation = "",
        format: _Format | tuple[float, float] = "",
        same: bool = False,
        duration: int = 0,
        transition: Incomplete | None = None,
    ) -> None: ...
    def header(self) -> None: ...
    def footer(self) -> None: ...
    def page_no(self) -> int: ...
    def set_draw_color(self, r: int, g: int = -1, b: int = -1) -> None: ...
    def set_fill_color(self, r: int, g: int = -1, b: int = -1) -> None: ...
    def set_text_color(self, r: int, g: int = -1, b: int = -1) -> None: ...
    def get_string_width(self, s: str, normalized: bool = False, markdown: bool = False) -> float: ...
    def set_line_width(self, width: float) -> None: ...
    def set_page_background(self, background) -> None: ...
    def drawing_context(self, debug_stream: Incomplete | None = None) -> _GeneratorContextManager[DrawingContext]: ...
    def new_path(
        self, x: float = 0, y: float = 0, paint_rule: PathPaintRule = ..., debug_stream: Incomplete | None = None
    ) -> _GeneratorContextManager[PaintedPath]: ...
    def draw_path(self, path: PaintedPath, debug_stream: Incomplete | None = None) -> None: ...
    def set_dash_pattern(self, dash: float = 0, gap: float = 0, phase: float = 0) -> None: ...
    def line(self, x1: float, y1: float, x2: float, y2: float) -> None: ...
    def polyline(
        self,
        point_list: list[tuple[float, float]],
        fill: bool = False,
        polygon: bool = False,
        style: RenderStyle | str | None = None,
    ) -> None: ...
    def polygon(
        self, point_list: list[tuple[float, float]], fill: bool = False, style: RenderStyle | str | None = None
    ) -> None: ...
    def dashed_line(self, x1, y1, x2, y2, dash_length: int = 1, space_length: int = 1) -> None: ...
    def rect(
        self,
        x: float,
        y: float,
        w: float,
        h: float,
        style: RenderStyle | str | None = None,
        round_corners: tuple[str, ...] | tuple[Corner, ...] | bool = False,
        corner_radius: float = 0,
    ) -> None: ...
    def ellipse(self, x: float, y: float, w: float, h: float, style: RenderStyle | str | None = None) -> None: ...
    def circle(self, x: float, y: float, r, style: RenderStyle | str | None = None) -> None: ...
    def regular_polygon(
        self,
        x: float,
        y: float,
        numSides: int,
        polyWidth: float,
        rotateDegrees: float = 0,
        style: RenderStyle | str | None = None,
    ): ...
    def star(
        self,
        x: float,
        y: float,
        r_in: float,
        r_out: float,
        corners: int,
        rotate_degrees: float = 0,
        style: RenderStyle | str | None = None,
    ): ...
    def arc(
        self,
        x: float,
        y: float,
        a: float,
        start_angle: float,
        end_angle: float,
        b: float | None = None,
        inclination: float = 0,
        clockwise: bool = False,
        start_from_center: bool = False,
        end_at_center: bool = False,
        style: RenderStyle | str | None = None,
    ) -> None: ...
    def solid_arc(
        self,
        x: float,
        y: float,
        a: float,
        start_angle: float,
        end_angle: float,
        b: float | None = None,
        inclination: float = 0,
        clockwise: bool = False,
        style: RenderStyle | str | None = None,
    ) -> None: ...
    def add_font(
        self,
        family: str | None = None,
        style: _FontStyle = "",
        fname: str | PurePath | None = None,
        uni: bool | Literal["DEPRECATED"] = "DEPRECATED",
    ) -> None: ...
    def set_font(self, family: str | None = None, style: _FontStyles = "", size: int = 0) -> None: ...
    def set_font_size(self, size: float) -> None: ...
    def set_char_spacing(self, spacing: float) -> None: ...
    def set_stretching(self, stretching: float) -> None: ...
    def set_fallback_fonts(self, fallback_fonts: Iterable[str], exact_match: bool = True) -> None: ...
    def add_link(self, y: float = 0, x: float = 0, page: int = -1, zoom: float | Literal["null"] = "null") -> int: ...
    def set_link(self, link, y: float = 0, x: float = 0, page: int = -1, zoom: float | Literal["null"] = "null") -> None: ...
    def link(
        self, x: float, y: float, w: float, h: float, link: str | int, alt_text: str | None = None, border_width: int = 0
    ) -> AnnotationDict: ...
    def embed_file(
        self,
        file_path: StrPath | None = None,
        bytes: bytes | None = None,
        basename: str | None = None,
        modification_date: datetime.datetime | None = None,
        *,
        creation_date: datetime.datetime | None = ...,
        desc: str = ...,
        compress: bool = ...,
        checksum: bool = ...,
    ) -> str: ...
    def file_attachment_annotation(
        self,
        file_path: StrPath,
        x: float,
        y: float,
        w: float = 1,
        h: float = 1,
        name: FileAttachmentAnnotationName | str | None = None,
        flags: Iterable[AnnotationFlag | str] = ...,
        *,
        bytes: bytes | None = ...,
        basename: str | None = ...,
        creation_date: datetime.datetime | None = ...,
        modification_date: datetime.datetime | None = ...,
        desc: str = ...,
        compress: bool = ...,
        checksum: bool = ...,
    ) -> AnnotationDict: ...
    def text_annotation(
        self,
        x: float,
        y: float,
        text: str,
        w: float = 1,
        h: float = 1,
        name: AnnotationName | str | None = None,
        flags: tuple[AnnotationFlag, ...] | tuple[str, ...] = ...,
    ) -> AnnotationDict: ...
    def free_text_annotation(
        self,
        text: str,
        x: float | None = None,
        y: float | None = None,
        w: float | None = None,
        h: float | None = None,
        flags: tuple[AnnotationFlag, ...] | tuple[str, ...] = ...,
    ) -> AnnotationDict: ...
    def add_action(self, action, x: float, y: float, w: float, h: float) -> None: ...
    def highlight(
        self,
        text: str,
        title: str = "",
        type: TextMarkupType | str = "Highlight",
        color: tuple[float, float, float] = (1, 1, 0),
        modification_time: datetime.datetime | None = None,
    ) -> _GeneratorContextManager[None]: ...
    add_highlight = highlight
    def add_text_markup_annotation(
        self,
        type: str,
        text: str,
        quad_points: Sequence[int],
        title: str = "",
        color: tuple[float, float, float] = (1, 1, 0),
        modification_time: datetime.datetime | None = None,
        page: int | None = None,
    ) -> AnnotationDict: ...
    def ink_annotation(
        self,
        coords: Iterable[Incomplete],
        contents: str = "",
        title: str = "",
        color: Sequence[float] = (1, 1, 0),
        border_width: int = 1,
    ) -> AnnotationDict: ...
    def text(self, x: float, y: float, text: str = "") -> None: ...
    def rotate(self, angle: float, x: float | None = None, y: float | None = None) -> None: ...
    def rotation(self, angle: float, x: float | None = None, y: float | None = None) -> _GeneratorContextManager[None]: ...
    def skew(
        self, ax: float = 0, ay: float = 0, x: float | None = None, y: float | None = None
    ) -> _GeneratorContextManager[None]: ...
    def mirror(self, origin, angle) -> Generator[None, None, None]: ...
    def local_context(
        self,
        font_family: Incomplete | None = None,
        font_style: Incomplete | None = None,
        font_size: Incomplete | None = None,
        line_width: Incomplete | None = None,
        draw_color: Incomplete | None = None,
        fill_color: Incomplete | None = None,
        text_color: Incomplete | None = None,
        dash_pattern: Incomplete | None = None,
        **kwargs,
    ) -> _GeneratorContextManager[None]: ...
    @property
    def accept_page_break(self) -> bool: ...
    def cell(
        self,
        w: float | None = None,
        h: float | None = None,
        text: str = "",
        border: bool | Literal[0, 1] | str = 0,
        ln: int | Literal["DEPRECATED"] = "DEPRECATED",
        align: str | Align = ...,
        fill: bool = False,
        link: str = "",
        center: bool = False,
        markdown: bool = False,
        new_x: XPos | str = ...,
        new_y: YPos | str = ...,
    ) -> bool: ...
    def get_fallback_font(self, char: str, style: str = "") -> str | None: ...
    def will_page_break(self, height: float) -> bool: ...
    def multi_cell(
        self,
        w: float,
        h: float | None = None,
        text: str = "",
        border: bool | Literal[0, 1] | str = 0,
        align: str | Align = ...,
        fill: bool = False,
        split_only: bool = False,
        link: str = "",
        ln: int | Literal["DEPRECATED"] = "DEPRECATED",
        max_line_height: float | None = None,
        markdown: bool = False,
        print_sh: bool = False,
        new_x: XPos | str = ...,
        new_y: YPos | str = ...,
        wrapmode: WrapMode = ...,
        dry_run: bool = False,
        output: MethodReturnValue | str | int = ...,
        center: bool = False,
        padding: int = 0,
    ): ...
    def write(
        self, h: float | None = None, text: str = "", link: str = "", print_sh: bool = False, wrapmode: WrapMode = ...
    ) -> bool: ...
    def text_columns(
        self,
        text: str | None = None,
        img: str | None = None,
        img_fill_width: bool = False,
        ncols: int = 1,
        gutter: float = 10,
        balance: bool = False,
        text_align: Align | _AlignLiteral = "LEFT",
        line_height: float = 1,
        l_margin: float | None = None,
        r_margin: float | None = None,
        print_sh: bool = False,
        wrapmode: WrapMode = ...,
        skip_leading_spaces: bool = False,
    ): ...
    def image(
        self,
        name: str | Image.Image | BytesIO | StrPath,
        x: float | Align | None = None,
        y: float | None = None,
        w: float = 0,
        h: float = 0,
        type: str = "",
        link: str = "",
        title: str | None = None,
        alt_text: str | None = None,
        dims: tuple[float, float] | None = None,
        keep_aspect_ratio: bool = False,
    ) -> RasterImageInfo | VectorImageInfo: ...
    @deprecated("Deprecated since 2.7.7; use fpdf.image_parsing.preload_image() instead")
    def preload_image(
        self, name: str | Image.Image | BytesIO, dims: tuple[float, float] | None = None
    ) -> tuple[str, Any, ImageInfo]: ...
    def ln(self, h: float | None = None) -> None: ...
    def get_x(self) -> float: ...
    def set_x(self, x: float) -> None: ...
    def get_y(self) -> float: ...
    def set_y(self, y: float) -> None: ...
    def set_xy(self, x: float, y: float) -> None: ...
    def normalize_text(self, text: str) -> str: ...
    def sign_pkcs12(
        self,
        pkcs_filepath: str,
        password: bytes | None = None,
        hashalgo: str = "sha256",
        contact_info: str | None = None,
        location: str | None = None,
        signing_time: datetime.datetime | None = None,
        reason: str | None = None,
        flags: tuple[AnnotationFlag, ...] = ...,
    ) -> None: ...
    def sign(
        self,
        key,
        cert,
        extra_certs: Sequence[Incomplete] = (),
        hashalgo: str = "sha256",
        contact_info: str | None = None,
        location: str | None = None,
        signing_time: datetime.datetime | None = None,
        reason: str | None = None,
        flags: tuple[AnnotationFlag, ...] = ...,
    ) -> None: ...
    def file_id(self) -> str: ...
    def interleaved2of5(self, text, x: float, y: float, w: float = 1, h: float = 10) -> None: ...
    def code39(self, text, x: float, y: float, w: float = 1.5, h: float = 5) -> None: ...
    def rect_clip(self, x: float, y: float, w: float, h: float) -> _GeneratorContextManager[None]: ...
    def elliptic_clip(self, x: float, y: float, w: float, h: float) -> _GeneratorContextManager[None]: ...
    def round_clip(self, x: float, y: float, r: float) -> _GeneratorContextManager[None]: ...
    def unbreakable(self) -> _GeneratorContextManager[FPDFRecorder]: ...
    def offset_rendering(self) -> _GeneratorContextManager[FPDFRecorder]: ...
    def insert_toc_placeholder(self, render_toc_function, pages: int = 1) -> None: ...
    def set_section_title_styles(
        self,
        level0: TitleStyle,
        level1: TitleStyle | None = None,
        level2: TitleStyle | None = None,
        level3: TitleStyle | None = None,
        level4: TitleStyle | None = None,
        level5: TitleStyle | None = None,
        level6: TitleStyle | None = None,
    ) -> None: ...
    def start_section(self, name: str, level: int = 0, strict: bool = True) -> None: ...
    def use_font_face(self, font_face: FontFace) -> _GeneratorContextManager[None]: ...
    def table(
        self,
        rows: Iterable[Incomplete] = (),
        *,
        align: str | Align = "CENTER",
        borders_layout: str | TableBordersLayout = ...,
        cell_fill_color: int | tuple[Incomplete, ...] | DeviceGray | DeviceRGB | None = None,
        cell_fill_mode: str | TableCellFillMode = ...,
        col_widths: int | tuple[int, ...] | None = None,
        first_row_as_headings: bool = True,
        headings_style: FontFace = ...,
        line_height: int | None = None,
        markdown: bool = False,
        text_align: str | Align = "JUSTIFY",
        width: int | None = None,
    ) -> _GeneratorContextManager[Table]: ...
    @overload
    def output(  # type: ignore[overload-overlap]
        self,
        name: Literal[""] | None = "",
        dest: Unused = "",
        linearize: bool = False,
        output_producer_class: Callable[[FPDF], OutputProducer] = ...,
    ) -> bytearray: ...
    @overload
    def output(
        self, name: str, dest: Unused = "", linearize: bool = False, output_producer_class: Callable[[FPDF], OutputProducer] = ...
    ) -> None: ...



from networkx.algorithms.bipartite.basic import *
from networkx.algorithms.bipartite.centrality import *
from networkx.algorithms.bipartite.cluster import *
from networkx.algorithms.bipartite.covering import *
from networkx.algorithms.bipartite.edgelist import *
from networkx.algorithms.bipartite.generators import *
from networkx.algorithms.bipartite.matching import *
from networkx.algorithms.bipartite.matrix import *
from networkx.algorithms.bipartite.projection import *
from networkx.algorithms.bipartite.redundancy import *
from networkx.algorithms.bipartite.spectral import *



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def percolation_centrality(
    G, attribute: str = "percolation", states: Incomplete | None = None, weight: Incomplete | None = None
): ...



from _typeshed import Incomplete

def add_operation(
    dn, attributes, auto_encode, schema: Incomplete | None = None, validator: Incomplete | None = None, check_names: bool = False
): ...
def add_request_to_dict(request): ...
def add_response_to_dict(response): ...



from typing import Literal

implementation: str
null: Literal[b""]

def from_bytes(octets, signed: bool = False): ...
def to_bytes(value, signed: bool = False, length: int = 0): ...
def bitLength(number): ...



from collections.abc import Callable
from typing import IO, Any, Generic, SupportsIndex, TypeVar, overload

_TB = TypeVar("_TB", bound=_BaseEntry)
_TP = TypeVar("_TP", bound=POFile)
_TM = TypeVar("_TM", bound=MOFile)

default_encoding: str

# wrapwidth: int
# encoding: str
# check_for_duplicates: bool
@overload
def pofile(pofile: str, *, klass: type[_TP], **kwargs: Any) -> _TP: ...
@overload
def pofile(pofile: str, **kwargs: Any) -> POFile: ...
@overload
def mofile(mofile: str, *, klass: type[_TM], **kwargs: Any) -> _TM: ...
@overload
def mofile(mofile: str, **kwargs: Any) -> MOFile: ...
def detect_encoding(file: bytes | str, binary_mode: bool = ...) -> str: ...
def escape(st: str) -> str: ...
def unescape(st: str) -> str: ...

class _BaseFile(list[_TB]):
    fpath: str
    wrapwidth: int
    encoding: str
    check_for_duplicates: bool
    header: str
    metadata: dict[str, str]
    metadata_is_fuzzy: bool
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __unicode__(self) -> str: ...
    def __contains__(self, entry: _TB) -> bool: ...  # type: ignore[override]  # AttributeError otherwise
    def __eq__(self, other: object) -> bool: ...
    def append(self, entry: _TB) -> None: ...
    def insert(self, index: SupportsIndex, entry: _TB) -> None: ...
    def metadata_as_entry(self) -> POEntry: ...
    def save(self, fpath: str | None = ..., repr_method: str = ..., newline: str | None = ...) -> None: ...
    def find(self, st: str, by: str = ..., include_obsolete_entries: bool = ..., msgctxt: bool = ...) -> _TB | None: ...
    def ordered_metadata(self) -> list[tuple[str, str]]: ...
    def to_binary(self) -> bytes: ...

class POFile(_BaseFile[POEntry]):
    def __unicode__(self) -> str: ...
    def save_as_mofile(self, fpath: str) -> None: ...
    def percent_translated(self) -> int: ...
    def translated_entries(self) -> list[POEntry]: ...
    def untranslated_entries(self) -> list[POEntry]: ...
    def fuzzy_entries(self) -> list[POEntry]: ...
    def obsolete_entries(self) -> list[POEntry]: ...
    def merge(self, refpot: POFile) -> None: ...

class MOFile(_BaseFile[MOEntry]):
    MAGIC: int
    MAGIC_SWAPPED: int
    magic_number: int | None
    version: int
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def save_as_pofile(self, fpath: str) -> None: ...
    def save(self, fpath: str | None = ...) -> None: ...  # type: ignore[override]  # binary file does not allow argument repr_method
    def percent_translated(self) -> int: ...
    def translated_entries(self) -> list[MOEntry]: ...
    def untranslated_entries(self) -> list[MOEntry]: ...
    def fuzzy_entries(self) -> list[MOEntry]: ...
    def obsolete_entries(self) -> list[MOEntry]: ...

class _BaseEntry:
    msgid: str
    msgstr: str
    msgid_plural: str
    msgstr_plural: dict[int, str]
    msgctxt: str
    obsolete: bool
    encoding: str
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __unicode__(self, wrapwidth: int = ...) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    @property
    def msgid_with_context(self) -> str: ...

class POEntry(_BaseEntry):
    comment: str
    tcomment: str
    occurrences: list[tuple[str, str]]
    flags: list[str]
    previous_msgctxt: str | None
    previous_msgid: str | None
    previous_msgid_plural: str | None
    linenum: int | None
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __unicode__(self, wrapwidth: int = ...) -> str: ...
    def __cmp__(self, other: POEntry) -> int: ...
    def __gt__(self, other: POEntry) -> bool: ...
    def __lt__(self, other: POEntry) -> bool: ...
    def __ge__(self, other: POEntry) -> bool: ...
    def __le__(self, other: POEntry) -> bool: ...
    def __eq__(self, other: POEntry) -> bool: ...  # type: ignore[override]
    def __ne__(self, other: POEntry) -> bool: ...  # type: ignore[override]
    def translated(self) -> bool: ...
    def merge(self, other: POEntry) -> None: ...
    @property
    def fuzzy(self) -> bool: ...
    @property
    def msgid_with_context(self) -> str: ...
    def __hash__(self) -> int: ...

class MOEntry(_BaseEntry):
    comment: str
    tcomment: str
    occurrences: list[tuple[str, str]]
    flags: list[str]
    previous_msgctxt: str | None
    previous_msgid: str | None
    previous_msgid_plural: str | None
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __hash__(self) -> int: ...

class _POFileParser(Generic[_TP]):
    fhandle: IO[str]
    instance: _TP
    transitions: dict[tuple[str, str], tuple[Callable[[], bool], str]]
    current_line: int
    current_entry: POEntry
    current_state: str
    current_token: str | None
    msgstr_index: int
    entry_obsolete: int
    def __init__(self, pofile: str, *args: Any, **kwargs: Any) -> None: ...
    def parse(self) -> _TP: ...
    def add(self, symbol: str, states: list[str], next_state: str) -> None: ...
    def process(self, symbol: str) -> None: ...
    def handle_he(self) -> bool: ...
    def handle_tc(self) -> bool: ...
    def handle_gc(self) -> bool: ...
    def handle_oc(self) -> bool: ...
    def handle_fl(self) -> bool: ...
    def handle_pp(self) -> bool: ...
    def handle_pm(self) -> bool: ...
    def handle_pc(self) -> bool: ...
    def handle_ct(self) -> bool: ...
    def handle_mi(self) -> bool: ...
    def handle_mp(self) -> bool: ...
    def handle_ms(self) -> bool: ...
    def handle_mx(self) -> bool: ...
    def handle_mc(self) -> bool: ...

class _MOFileParser(Generic[_TM]):
    fhandle: IO[bytes]
    instance: _TM
    def __init__(self, mofile: str, *args: Any, **kwargs: Any) -> None: ...
    def __del__(self) -> None: ...
    def parse(self) -> _TM: ...



from _typeshed import Incomplete

class StackResources:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        api_version: Incomplete | None = None,
        resource_id: Incomplete | None = None,
        kind: Incomplete | None = None,
        template_meta_name: Incomplete | None = None,
        associations: Incomplete | None = None,
        links: Incomplete | None = None,
    ) -> None: ...
    @property
    def api_version(self): ...
    @api_version.setter
    def api_version(self, api_version) -> None: ...
    @property
    def resource_id(self): ...
    @resource_id.setter
    def resource_id(self, resource_id) -> None: ...
    @property
    def kind(self): ...
    @kind.setter
    def kind(self, kind) -> None: ...
    @property
    def template_meta_name(self): ...
    @template_meta_name.setter
    def template_meta_name(self, template_meta_name) -> None: ...
    @property
    def associations(self): ...
    @associations.setter
    def associations(self, associations) -> None: ...
    @property
    def links(self): ...
    @links.setter
    def links(self, links) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from decimal import Decimal
from typing import Any, Literal

from markupsafe import Markup
from wtforms.fields import Field, FormField, SelectFieldBase, StringField
from wtforms.fields.choices import _Option

__all__ = (
    "CheckboxInput",
    "ColorInput",
    "DateInput",
    "DateTimeInput",
    "DateTimeLocalInput",
    "EmailInput",
    "FileInput",
    "HiddenInput",
    "ListWidget",
    "MonthInput",
    "NumberInput",
    "Option",
    "PasswordInput",
    "RadioInput",
    "RangeInput",
    "SearchInput",
    "Select",
    "SubmitInput",
    "TableWidget",
    "TextArea",
    "TextInput",
    "TelInput",
    "TimeInput",
    "URLInput",
    "WeekInput",
)

def html_params(**kwargs: object) -> str: ...

class ListWidget:
    html_tag: Literal["ul", "ol"]
    prefix_label: bool
    def __init__(self, html_tag: Literal["ul", "ol"] = "ul", prefix_label: bool = True) -> None: ...
    # any iterable field is fine, since people might define iterable fields
    # that are not derived from FieldList, we just punt and accept any field
    # with Intersection we could be more specific
    def __call__(self, field: Field, **kwargs: object) -> Markup: ...

class TableWidget:
    with_table_tag: bool
    def __init__(self, with_table_tag: bool = True) -> None: ...
    def __call__(self, field: FormField[Any], **kwargs: object) -> Markup: ...

class Input:
    validation_attrs: list[str]
    input_type: str
    def __init__(self, input_type: str | None = None) -> None: ...
    def __call__(self, field: Field, **kwargs: object) -> Markup: ...
    @staticmethod
    def html_params(**kwargs: object) -> str: ...

class TextInput(Input): ...

class PasswordInput(Input):
    hide_value: bool
    def __init__(self, hide_value: bool = True) -> None: ...

class HiddenInput(Input):
    field_flags: dict[str, Any]

class CheckboxInput(Input): ...
class RadioInput(Input): ...

class FileInput(Input):
    multiple: bool
    def __init__(self, multiple: bool = False) -> None: ...

class SubmitInput(Input): ...

class TextArea:
    validation_attrs: list[str]
    def __call__(self, field: StringField, **kwargs: object) -> Markup: ...

class Select:
    validation_attrs: list[str]
    multiple: bool
    def __init__(self, multiple: bool = False) -> None: ...
    def __call__(self, field: SelectFieldBase, **kwargs: object) -> Markup: ...
    @classmethod
    def render_option(cls, value: object, label: str, selected: bool, **kwargs: object) -> Markup: ...

class Option:
    def __call__(self, field: _Option, **kwargs: object) -> Markup: ...

class SearchInput(Input): ...
class TelInput(Input): ...
class URLInput(Input): ...
class EmailInput(Input): ...
class DateTimeInput(Input): ...
class DateInput(Input): ...
class MonthInput(Input): ...
class WeekInput(Input): ...
class TimeInput(Input): ...
class DateTimeLocalInput(Input): ...

class NumberInput(Input):
    step: Decimal | float | str | None
    min: Decimal | float | str | None
    max: Decimal | float | str | None
    def __init__(
        self,
        step: Decimal | float | str | None = None,
        min: Decimal | float | str | None = None,
        max: Decimal | float | str | None = None,
    ) -> None: ...

class RangeInput(Input):
    # maybe we should allow any str for this
    step: Decimal | float | str | None
    def __init__(self, step: Decimal | float | str | None = None) -> None: ...

class ColorInput(Input): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Brazil(WesternCalendar):
    FIXED_HOLIDAYS: Incomplete
    include_sao_jose: ClassVar[bool]
    sao_jose_label: ClassVar[str]
    include_sao_pedro: ClassVar[bool]
    sao_pedro_label: ClassVar[str]
    include_sao_joao: ClassVar[bool]
    sao_joao_label: ClassVar[str]
    include_labour_day: ClassVar[bool]
    include_servidor_publico: ClassVar[bool]
    servidor_publico_label: ClassVar[str]
    include_consciencia_negra: ClassVar[bool]
    consciencia_negra_day: Incomplete
    consciencia_negra_label: ClassVar[str]
    include_easter_sunday: ClassVar[bool]
    include_immaculate_conception: ClassVar[bool]
    immaculate_conception_label: ClassVar[str]
    def get_variable_days(self, year): ...

class BrazilAcre(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilAlagoas(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_sao_pedro: ClassVar[bool]
    include_sao_joao: ClassVar[bool]
    include_consciencia_negra: ClassVar[bool]

class BrazilAmapa(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_sao_jose: ClassVar[bool]
    sao_jose_label: ClassVar[str]
    include_consciencia_negra: ClassVar[bool]

class BrazilAmazonas(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_consciencia_negra: ClassVar[bool]
    include_immaculate_conception: ClassVar[bool]

class BrazilBahia(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilCeara(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_sao_jose: ClassVar[bool]

class BrazilDistritoFederal(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilEspiritoSanto(Brazil):
    include_servidor_publico: ClassVar[bool]

class BrazilGoias(Brazil):
    include_servidor_publico: ClassVar[bool]

class BrazilMaranhao(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_immaculate_conception: ClassVar[bool]

class BrazilMinasGerais(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilMatoGrosso(Brazil):
    include_consciencia_negra: ClassVar[bool]
    consciencia_negra_day: Incomplete

class BrazilMatoGrossoDoSul(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilPara(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_immaculate_conception: ClassVar[bool]

class BrazilParaiba(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilPernambuco(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_sao_joao: ClassVar[bool]

class BrazilPiaui(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilParana(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilRioDeJaneiro(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_fat_tuesday: ClassVar[bool]
    fat_tuesday_label: ClassVar[str]
    include_servidor_publico: ClassVar[bool]
    servidor_publico_label: ClassVar[str]
    include_consciencia_negra: ClassVar[bool]
    consciencia_negra_label: ClassVar[str]
    include_immaculate_conception: ClassVar[bool]
    def get_dia_do_comercio(self, year): ...
    def get_variable_days(self, year): ...

class BrazilRioGrandeDoNorte(Brazil):
    FIXED_HOLIDAYS: Incomplete
    include_sao_pedro: ClassVar[bool]
    sao_pedro_label: ClassVar[str]

class BrazilRioGrandeDoSul(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilRondonia(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilRoraima(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilSantaCatarina(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilSaoPauloState(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilSaoPauloCity(BrazilSaoPauloState):
    FIXED_HOLIDAYS: Incomplete
    include_fat_tuesday: ClassVar[bool]
    fat_tuesday_label: ClassVar[str]
    include_easter_sunday: ClassVar[bool]
    include_corpus_christi: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    good_friday_label: ClassVar[str]
    include_consciencia_negra: ClassVar[bool]
    consciencia_negra_label: ClassVar[str]

class BrazilSergipe(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilTocantins(Brazil):
    FIXED_HOLIDAYS: Incomplete

class BrazilVitoriaCity(BrazilEspiritoSanto):
    FIXED_HOLIDAYS: Incomplete
    include_corpus_christi: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    good_friday_label: ClassVar[str]

class BrazilVilaVelhaCity(BrazilEspiritoSanto):
    FIXED_HOLIDAYS: Incomplete

class BrazilCariacicaCity(BrazilEspiritoSanto):
    FIXED_HOLIDAYS: Incomplete
    include_corpus_christi: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    good_friday_label: ClassVar[str]
    include_sao_joao: ClassVar[bool]
    sao_joao_label: ClassVar[str]

class BrazilGuarapariCity(BrazilEspiritoSanto):
    FIXED_HOLIDAYS: Incomplete
    include_sao_pedro: ClassVar[bool]
    include_consciencia_negra: ClassVar[bool]
    consciencia_negra_day: Incomplete
    include_immaculate_conception: ClassVar[bool]

class BrazilSerraCity(BrazilEspiritoSanto):
    FIXED_HOLIDAYS: Incomplete
    include_fat_tuesday: ClassVar[bool]
    fat_tuesday_label: ClassVar[str]
    include_ash_wednesday: ClassVar[bool]
    ash_wednesday_label: ClassVar[str]
    include_good_friday: ClassVar[bool]
    good_friday_label: ClassVar[str]
    include_sao_pedro: ClassVar[bool]
    include_immaculate_conception: ClassVar[bool]
    def get_variable_days(self, year): ...

class BrazilRioBrancoCity(BrazilAcre):
    FIXED_HOLIDAYS: Incomplete

class BrazilMaceioCity(BrazilAlagoas):
    FIXED_HOLIDAYS: Incomplete

class BrazilManausCity(BrazilAmazonas):
    FIXED_HOLIDAYS: Incomplete

class BrazilMacapaCity(BrazilAmapa):
    FIXED_HOLIDAYS: Incomplete

class BrazilSalvadorCity(BrazilBahia):
    FIXED_HOLIDAYS: Incomplete

class BrazilFortalezaCity(BrazilCeara):
    FIXED_HOLIDAYS: Incomplete

class BrazilGoianiaCity(BrazilGoias):
    FIXED_HOLIDAYS: Incomplete

class BrazilBeloHorizonteCity(BrazilMinasGerais):
    FIXED_HOLIDAYS: Incomplete

class BrazilCampoGrandeCity(BrazilMatoGrossoDoSul):
    FIXED_HOLIDAYS: Incomplete

class BrazilCuiabaCity(BrazilMatoGrosso):
    FIXED_HOLIDAYS: Incomplete
    include_easter_sunday: ClassVar[bool]
    include_corpus_christi: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    good_friday_label: ClassVar[str]

class BrazilBelemCity(BrazilPara):
    FIXED_HOLIDAYS: Incomplete

class BrazilJoaoPessoaCity(BrazilParaiba):
    FIXED_HOLIDAYS: Incomplete

class BrazilRecifeCity(BrazilPernambuco):
    FIXED_HOLIDAYS: Incomplete

class BrazilTeresinaCity(BrazilPiaui):
    FIXED_HOLIDAYS: Incomplete

class BrazilCuritibaCity(BrazilParana):
    FIXED_HOLIDAYS: Incomplete

class BrazilNatalCity(BrazilRioGrandeDoNorte):
    FIXED_HOLIDAYS: Incomplete

class BrazilPortoVelhoCity(BrazilRondonia):
    FIXED_HOLIDAYS: Incomplete

class BrazilBoaVistaCity(BrazilRoraima):
    FIXED_HOLIDAYS: Incomplete

class BrazilPortoAlegreCity(BrazilRioGrandeDoSul):
    FIXED_HOLIDAYS: Incomplete

class BrazilChapecoCity(BrazilSantaCatarina):
    FIXED_HOLIDAYS: Incomplete

class BrazilFlorianopolisCity(BrazilSantaCatarina):
    FIXED_HOLIDAYS: Incomplete

class BrazilJoinvilleCity(BrazilSantaCatarina):
    FIXED_HOLIDAYS: Incomplete

class BrazilAracajuCity(BrazilSergipe):
    FIXED_HOLIDAYS: Incomplete

class BrazilSorocabaCity(BrazilSaoPauloState):
    FIXED_HOLIDAYS: Incomplete

class BrazilPalmasCity(BrazilTocantins):
    FIXED_HOLIDAYS: Incomplete

class BrazilBankCalendar(Brazil):
    include_fat_tuesday: ClassVar[bool]
    fat_tuesday_label: ClassVar[str]
    include_good_friday: ClassVar[bool]
    include_ash_wednesday: ClassVar[bool]
    include_corpus_christi: ClassVar[bool]
    include_easter_sunday: ClassVar[bool]
    def get_last_day_of_year_for_only_internal_bank_trans(self, year): ...
    def get_variable_days(self, year): ...
    def find_following_working_day(self, day): ...

IBGE_TUPLE: Incomplete
IBGE_REGISTER: Incomplete



from collections.abc import Callable, Collection, Sequence
from typing import Any
from typing_extensions import Self

from wtforms.fields.core import Field, _Filter, _FormT, _Validator, _Widget
from wtforms.form import BaseForm
from wtforms.meta import DefaultMeta, _SupportsGettextAndNgettext

class BooleanField(Field):
    data: bool
    default: bool | Callable[[], bool] | None
    false_values: Collection[Any]
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        false_values: Collection[Any] | None = None,
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: bool | Callable[[], bool] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class StringField(Field):
    data: str | None
    default: str | Callable[[], str] | None
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: str | Callable[[], str] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class TextAreaField(StringField): ...
class PasswordField(StringField): ...
class FileField(Field): ...

class MultipleFileField(FileField):
    data: list[Any]

class HiddenField(StringField): ...
class SubmitField(BooleanField): ...
class SearchField(StringField): ...
class TelField(StringField): ...
class URLField(StringField): ...
class EmailField(StringField): ...
class ColorField(StringField): ...



import sys
from collections.abc import Iterable
from select import error as error
from typing import Any

def select(
    rlist: Iterable[Any], wlist: Iterable[Any], xlist: Iterable[Any], timeout: float | None = None
) -> tuple[list[Any], list[Any], list[Any]]: ...

if sys.platform != "win32":
    from select import poll as poll

    __all__ = ["error", "poll", "select"]
else:
    __all__ = ["error", "select"]



from _typeshed import Incomplete

integer_types = int
text_type = str
binary_type = bytes

class Generator:
    dict: dict[Incomplete, Incomplete]
    def __init__(self, dict) -> None: ...
    def generate(self): ...



from collections.abc import Callable
from logging import Logger

from ._types import Reader

logger: Logger

class WrongBox(Exception): ...
class NoParser(Exception): ...
class BoxVersion(Exception): ...
class BadSize(Exception): ...

class Box:
    version: int
    minor_version: int
    item_count: int
    size: int
    after: int
    pos: int
    compat: list[bytes]
    base_offset: int
    subs: dict[str, Box]
    locs: dict[int, list[tuple[int, int]]]
    exif_infe: Box | None
    item_id: int
    item_type: bytes
    item_name: bytes
    item_protection_index: int
    major_brand: bytes
    offset_size: int
    length_size: int
    base_offset_size: int
    index_size: int
    flags: int
    name: str
    def __init__(self, name: str) -> None: ...
    def set_sizes(self, offset: int, length: int, base_offset: int, index: int) -> None: ...
    def set_full(self, vflags: int) -> None: ...

class HEICExifFinder:
    file_handle: Reader
    def __init__(self, file_handle: Reader) -> None: ...
    def get(self, nbytes: int) -> bytes: ...
    def get16(self) -> int: ...
    def get32(self) -> int: ...
    def get64(self) -> int: ...
    def get_int4x2(self) -> tuple[int, int]: ...
    def get_int(self, size: int) -> int: ...
    def get_string(self) -> bytes: ...
    def next_box(self) -> Box: ...
    def get_full(self, box: Box) -> None: ...
    def skip(self, box: Box) -> None: ...
    def expect_parse(self, name: str) -> Box: ...
    def get_parser(self, box: Box) -> Callable[[Box], None]: ...
    def parse_box(self, box: Box) -> Box: ...
    def find_exif(self) -> tuple[int, bytes]: ...



from typing import overload

from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    SearchableAPIResource as SearchableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
)

class PaymentIntent(CreateableAPIResource, ListableAPIResource, SearchableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def cancel(self, idempotency_key: str | None = None, **params): ...
    def capture(self, idempotency_key: str | None = None, **params): ...
    @overload
    @classmethod
    def confirm(
        cls, intent: str, api_key: str | None = ..., stripe_version: str | None = ..., stripe_account: str | None = ..., **params
    ): ...
    @overload
    @classmethod
    def confirm(cls, idempotency_key: str | None = None, **params): ...



from collections.abc import Callable, Iterable
from typing import Any, TypedDict, TypeVar
from typing_extensions import NotRequired

TYPES_MAP: dict[str, str]
REVERSE_TYPES_MAP: dict[str, tuple[str, ...]]

class _Signature(TypedDict):
    types: list[str]
    variadic: NotRequired[bool]

_F = TypeVar("_F", bound=Callable[..., Any])

def signature(*arguments: _Signature) -> Callable[[_F], _F]: ...

class FunctionRegistry(type):
    def __init__(cls, name, bases, attrs) -> None: ...

class Functions(metaclass=FunctionRegistry):
    FUNCTION_TABLE: Any
    # resolved_args and return value are the *args and return of a function called by name
    def call_function(self, function_name: str, resolved_args: Iterable[Any]) -> Any: ...



from typing import Any

from .endpoints.base import BaseEndpoint as BaseEndpoint, catch_errors_and_unavailability as catch_errors_and_unavailability
from .errors import (
    FatalClientError as FatalClientError,
    OAuth2Error as OAuth2Error,
    ServerError as ServerError,
    TemporarilyUnavailableError as TemporarilyUnavailableError,
)

log: Any



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Score(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ScoreType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ScoreTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Score._ScoreType.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        Curriculum: Score._ScoreType.ValueType  # 1
        """map generated score (from curriculum maps with special scoring)"""
        Melee: Score._ScoreType.ValueType  # 2
        """summation of in-progress and current units/buildings value + minerals + vespene"""

    class ScoreType(_ScoreType, metaclass=_ScoreTypeEnumTypeWrapper): ...
    Curriculum: Score.ScoreType.ValueType  # 1
    """map generated score (from curriculum maps with special scoring)"""
    Melee: Score.ScoreType.ValueType  # 2
    """summation of in-progress and current units/buildings value + minerals + vespene"""

    SCORE_TYPE_FIELD_NUMBER: builtins.int
    SCORE_FIELD_NUMBER: builtins.int
    SCORE_DETAILS_FIELD_NUMBER: builtins.int
    score_type: global___Score.ScoreType.ValueType
    score: builtins.int
    """Note: check score_type to know whether this is a melee score or curriculum score"""
    @property
    def score_details(self) -> global___ScoreDetails: ...
    def __init__(
        self,
        *,
        score_type: global___Score.ScoreType.ValueType | None = ...,
        score: builtins.int | None = ...,
        score_details: global___ScoreDetails | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["score", b"score", "score_details", b"score_details", "score_type", b"score_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["score", b"score", "score_details", b"score_details", "score_type", b"score_type"]) -> None: ...

global___Score = Score

@typing_extensions.final
class CategoryScoreDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NONE_FIELD_NUMBER: builtins.int
    ARMY_FIELD_NUMBER: builtins.int
    ECONOMY_FIELD_NUMBER: builtins.int
    TECHNOLOGY_FIELD_NUMBER: builtins.int
    UPGRADE_FIELD_NUMBER: builtins.int
    none: builtins.float
    """Used when no other category is configured in game data"""
    army: builtins.float
    economy: builtins.float
    technology: builtins.float
    upgrade: builtins.float
    def __init__(
        self,
        *,
        none: builtins.float | None = ...,
        army: builtins.float | None = ...,
        economy: builtins.float | None = ...,
        technology: builtins.float | None = ...,
        upgrade: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["army", b"army", "economy", b"economy", "none", b"none", "technology", b"technology", "upgrade", b"upgrade"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["army", b"army", "economy", b"economy", "none", b"none", "technology", b"technology", "upgrade", b"upgrade"]) -> None: ...

global___CategoryScoreDetails = CategoryScoreDetails

@typing_extensions.final
class VitalScoreDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LIFE_FIELD_NUMBER: builtins.int
    SHIELDS_FIELD_NUMBER: builtins.int
    ENERGY_FIELD_NUMBER: builtins.int
    life: builtins.float
    shields: builtins.float
    energy: builtins.float
    def __init__(
        self,
        *,
        life: builtins.float | None = ...,
        shields: builtins.float | None = ...,
        energy: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["energy", b"energy", "life", b"life", "shields", b"shields"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["energy", b"energy", "life", b"life", "shields", b"shields"]) -> None: ...

global___VitalScoreDetails = VitalScoreDetails

@typing_extensions.final
class ScoreDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IDLE_PRODUCTION_TIME_FIELD_NUMBER: builtins.int
    IDLE_WORKER_TIME_FIELD_NUMBER: builtins.int
    TOTAL_VALUE_UNITS_FIELD_NUMBER: builtins.int
    TOTAL_VALUE_STRUCTURES_FIELD_NUMBER: builtins.int
    KILLED_VALUE_UNITS_FIELD_NUMBER: builtins.int
    KILLED_VALUE_STRUCTURES_FIELD_NUMBER: builtins.int
    COLLECTED_MINERALS_FIELD_NUMBER: builtins.int
    COLLECTED_VESPENE_FIELD_NUMBER: builtins.int
    COLLECTION_RATE_MINERALS_FIELD_NUMBER: builtins.int
    COLLECTION_RATE_VESPENE_FIELD_NUMBER: builtins.int
    SPENT_MINERALS_FIELD_NUMBER: builtins.int
    SPENT_VESPENE_FIELD_NUMBER: builtins.int
    FOOD_USED_FIELD_NUMBER: builtins.int
    KILLED_MINERALS_FIELD_NUMBER: builtins.int
    KILLED_VESPENE_FIELD_NUMBER: builtins.int
    LOST_MINERALS_FIELD_NUMBER: builtins.int
    LOST_VESPENE_FIELD_NUMBER: builtins.int
    FRIENDLY_FIRE_MINERALS_FIELD_NUMBER: builtins.int
    FRIENDLY_FIRE_VESPENE_FIELD_NUMBER: builtins.int
    USED_MINERALS_FIELD_NUMBER: builtins.int
    USED_VESPENE_FIELD_NUMBER: builtins.int
    TOTAL_USED_MINERALS_FIELD_NUMBER: builtins.int
    TOTAL_USED_VESPENE_FIELD_NUMBER: builtins.int
    TOTAL_DAMAGE_DEALT_FIELD_NUMBER: builtins.int
    TOTAL_DAMAGE_TAKEN_FIELD_NUMBER: builtins.int
    TOTAL_HEALED_FIELD_NUMBER: builtins.int
    CURRENT_APM_FIELD_NUMBER: builtins.int
    CURRENT_EFFECTIVE_APM_FIELD_NUMBER: builtins.int
    idle_production_time: builtins.float
    """Sum of time any available structure able to produce a unit is not. The time stacks, as in, three idle barracks will increase idle_production_time three times quicker than just one."""
    idle_worker_time: builtins.float
    """Sum of time any worker is not mining. Note a worker building is not idle and three idle workers will increase this value three times quicker than just one."""
    total_value_units: builtins.float
    """Sum of minerals and vespene spent on completed units."""
    total_value_structures: builtins.float
    """Sum of minerals and vespene spent on completed structures."""
    killed_value_units: builtins.float
    """Sum of minerals and vespene of units, belonging to the opponent, that the player has destroyed."""
    killed_value_structures: builtins.float
    """Sum of minerals and vespene of structures, belonging to the opponent, that the player has destroyed."""
    collected_minerals: builtins.float
    """Sum of minerals collected by the player."""
    collected_vespene: builtins.float
    """Sum of vespene collected by the player."""
    collection_rate_minerals: builtins.float
    """Estimated income of minerals over the next minute based on the players current income. The unit is minerals per minute."""
    collection_rate_vespene: builtins.float
    """Estimated income of vespene over the next minute based on the players current income. The unit is vespene per minute."""
    spent_minerals: builtins.float
    """Sum of spent minerals at the moment it is spent. For example, this number is incremented by 50 the moment an scv is queued in a command center.  It is decremented by 50 if that unit is canceled."""
    spent_vespene: builtins.float
    """Sum of spent vespene at the moment it is spent. For example, this number is incremented by 50 when a reaper is queued but decremented by 50 if it is canceled."""
    @property
    def food_used(self) -> global___CategoryScoreDetails:
        """The following entries contains floating point values for the following catgories:
          none - There is no category defined in game data.
          army - This category includes all military units but not workers.
          economy - This category contains town halls, supply structures, vespene buildings and workers.
          technology - This category is any structure that produces units or upgrades, Barracks and Engineering Bays both fall in this category, for example.
          upgrade - This category is upgrades such as warp gate or weapons upgrades.

        Sum of food, or supply, utilized in the categories above.
        """
    @property
    def killed_minerals(self) -> global___CategoryScoreDetails:
        """Sum of enemies catagories destroyed in minerals."""
    @property
    def killed_vespene(self) -> global___CategoryScoreDetails:
        """Sum of enemies catagories destroyed in vespene."""
    @property
    def lost_minerals(self) -> global___CategoryScoreDetails:
        """ Sum of lost minerals for the player in each category."""
    @property
    def lost_vespene(self) -> global___CategoryScoreDetails:
        """Sum of lost vespene for the player in each category."""
    @property
    def friendly_fire_minerals(self) -> global___CategoryScoreDetails:
        """Sum of the lost minerals via destroying the players own units/buildings."""
    @property
    def friendly_fire_vespene(self) -> global___CategoryScoreDetails:
        """Sum of the lost vespene via destroying the players own units/buildings."""
    @property
    def used_minerals(self) -> global___CategoryScoreDetails:
        """Sum of used minerals for the player in each category for each existing unit or upgrade. Therefore if a unit died worth 50 mierals this number will be decremented by 50."""
    @property
    def used_vespene(self) -> global___CategoryScoreDetails:
        """Sum of used vespene for the player in each category. Therefore if a unit died worth 50 vespene this number will be decremented by 50."""
    @property
    def total_used_minerals(self) -> global___CategoryScoreDetails:
        """Sum of used minerals throughout the entire game for each category. Unliked used_minerals, this value is never decremented."""
    @property
    def total_used_vespene(self) -> global___CategoryScoreDetails:
        """Sum of used vespene throughout the entire game for each category. Unliked used_vespene, this value is never decremented."""
    @property
    def total_damage_dealt(self) -> global___VitalScoreDetails:
        """Sum of damage dealt to the player's opponent for each category."""
    @property
    def total_damage_taken(self) -> global___VitalScoreDetails:
        """Sum of damage taken by the player for each category."""
    @property
    def total_healed(self) -> global___VitalScoreDetails:
        """Sum of health healed by the player. Note that technology can be healed (by queens) or repaired (by scvs)."""
    current_apm: builtins.float
    """Recent raw APM."""
    current_effective_apm: builtins.float
    """Recent effective APM."""
    def __init__(
        self,
        *,
        idle_production_time: builtins.float | None = ...,
        idle_worker_time: builtins.float | None = ...,
        total_value_units: builtins.float | None = ...,
        total_value_structures: builtins.float | None = ...,
        killed_value_units: builtins.float | None = ...,
        killed_value_structures: builtins.float | None = ...,
        collected_minerals: builtins.float | None = ...,
        collected_vespene: builtins.float | None = ...,
        collection_rate_minerals: builtins.float | None = ...,
        collection_rate_vespene: builtins.float | None = ...,
        spent_minerals: builtins.float | None = ...,
        spent_vespene: builtins.float | None = ...,
        food_used: global___CategoryScoreDetails | None = ...,
        killed_minerals: global___CategoryScoreDetails | None = ...,
        killed_vespene: global___CategoryScoreDetails | None = ...,
        lost_minerals: global___CategoryScoreDetails | None = ...,
        lost_vespene: global___CategoryScoreDetails | None = ...,
        friendly_fire_minerals: global___CategoryScoreDetails | None = ...,
        friendly_fire_vespene: global___CategoryScoreDetails | None = ...,
        used_minerals: global___CategoryScoreDetails | None = ...,
        used_vespene: global___CategoryScoreDetails | None = ...,
        total_used_minerals: global___CategoryScoreDetails | None = ...,
        total_used_vespene: global___CategoryScoreDetails | None = ...,
        total_damage_dealt: global___VitalScoreDetails | None = ...,
        total_damage_taken: global___VitalScoreDetails | None = ...,
        total_healed: global___VitalScoreDetails | None = ...,
        current_apm: builtins.float | None = ...,
        current_effective_apm: builtins.float | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["collected_minerals", b"collected_minerals", "collected_vespene", b"collected_vespene", "collection_rate_minerals", b"collection_rate_minerals", "collection_rate_vespene", b"collection_rate_vespene", "current_apm", b"current_apm", "current_effective_apm", b"current_effective_apm", "food_used", b"food_used", "friendly_fire_minerals", b"friendly_fire_minerals", "friendly_fire_vespene", b"friendly_fire_vespene", "idle_production_time", b"idle_production_time", "idle_worker_time", b"idle_worker_time", "killed_minerals", b"killed_minerals", "killed_value_structures", b"killed_value_structures", "killed_value_units", b"killed_value_units", "killed_vespene", b"killed_vespene", "lost_minerals", b"lost_minerals", "lost_vespene", b"lost_vespene", "spent_minerals", b"spent_minerals", "spent_vespene", b"spent_vespene", "total_damage_dealt", b"total_damage_dealt", "total_damage_taken", b"total_damage_taken", "total_healed", b"total_healed", "total_used_minerals", b"total_used_minerals", "total_used_vespene", b"total_used_vespene", "total_value_structures", b"total_value_structures", "total_value_units", b"total_value_units", "used_minerals", b"used_minerals", "used_vespene", b"used_vespene"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["collected_minerals", b"collected_minerals", "collected_vespene", b"collected_vespene", "collection_rate_minerals", b"collection_rate_minerals", "collection_rate_vespene", b"collection_rate_vespene", "current_apm", b"current_apm", "current_effective_apm", b"current_effective_apm", "food_used", b"food_used", "friendly_fire_minerals", b"friendly_fire_minerals", "friendly_fire_vespene", b"friendly_fire_vespene", "idle_production_time", b"idle_production_time", "idle_worker_time", b"idle_worker_time", "killed_minerals", b"killed_minerals", "killed_value_structures", b"killed_value_structures", "killed_value_units", b"killed_value_units", "killed_vespene", b"killed_vespene", "lost_minerals", b"lost_minerals", "lost_vespene", b"lost_vespene", "spent_minerals", b"spent_minerals", "spent_vespene", b"spent_vespene", "total_damage_dealt", b"total_damage_dealt", "total_damage_taken", b"total_damage_taken", "total_healed", b"total_healed", "total_used_minerals", b"total_used_minerals", "total_used_vespene", b"total_used_vespene", "total_value_structures", b"total_value_structures", "total_value_units", b"total_value_units", "used_minerals", b"used_minerals", "used_vespene", b"used_vespene"]) -> None: ...

global___ScoreDetails = ScoreDetails



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def is_biconnected(G): ...
@_dispatch
def biconnected_component_edges(G) -> Generator[Incomplete, Incomplete, None]: ...
@_dispatch
def biconnected_components(G) -> Generator[Incomplete, None, None]: ...
@_dispatch
def articulation_points(G) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

class Gcp(VaultApiBase):
    def configure(
        self,
        credentials: Incomplete | None = None,
        ttl: Incomplete | None = None,
        max_ttl: Incomplete | None = None,
        mount_point="gcp",
    ): ...
    def rotate_root_credentials(self, mount_point="gcp"): ...
    def read_config(self, mount_point="gcp"): ...
    def create_or_update_roleset(
        self,
        name,
        project,
        bindings,
        secret_type: Incomplete | None = None,
        token_scopes: Incomplete | None = None,
        mount_point="gcp",
    ): ...
    def rotate_roleset_account(self, name, mount_point="gcp"): ...
    def rotate_roleset_account_key(self, name, mount_point="gcp"): ...
    def read_roleset(self, name, mount_point="gcp"): ...
    def list_rolesets(self, mount_point="gcp"): ...
    def delete_roleset(self, name, mount_point="gcp"): ...
    def generate_oauth2_access_token(self, roleset, mount_point="gcp"): ...
    def generate_service_account_key(
        self,
        roleset,
        key_algorithm: str = "KEY_ALG_RSA_2048",
        key_type: str = "TYPE_GOOGLE_CREDENTIALS_FILE",
        method: str = "POST",
        mount_point="gcp",
    ): ...
    def create_or_update_static_account(
        self,
        name,
        service_account_email,
        bindings: Incomplete | None = None,
        secret_type: Incomplete | None = None,
        token_scopes: Incomplete | None = None,
        mount_point="gcp",
    ): ...
    def rotate_static_account_key(self, name, mount_point="gcp"): ...
    def read_static_account(self, name, mount_point="gcp"): ...
    def list_static_accounts(self, mount_point="gcp"): ...
    def delete_static_account(self, name, mount_point="gcp"): ...
    def generate_static_account_oauth2_access_token(self, name, mount_point="gcp"): ...
    def generate_static_account_service_account_key(
        self,
        name,
        key_algorithm: str = "KEY_ALG_RSA_2048",
        key_type: str = "TYPE_GOOGLE_CREDENTIALS_FILE",
        method: str = "POST",
        mount_point="gcp",
    ): ...
    def create_or_update_impersonated_account(
        self,
        name,
        service_account_email,
        token_scopes: Incomplete | None = None,
        ttl: Incomplete | None = None,
        mount_point="gcp",
    ): ...
    def read_impersonated_account(self, name, mount_point="gcp"): ...
    def list_impersonated_accounts(self, mount_point="gcp"): ...
    def delete_impersonated_account(self, name, mount_point="gcp"): ...
    def generate_impersonated_account_oauth2_access_token(self, name, mount_point="gcp"): ...



from networkx.utils.backends import _dispatch

@_dispatch
def tensor_product(G, H): ...
@_dispatch
def cartesian_product(G, H): ...
@_dispatch
def lexicographic_product(G, H): ...
@_dispatch
def strong_product(G, H): ...
@_dispatch
def power(G, k): ...
@_dispatch
def rooted_product(G, H, root): ...
@_dispatch
def corona_product(G, H): ...



from _typeshed import Unused
from typing import ClassVar, Literal

from openpyxl import _Decodable, _VisibilityType
from openpyxl.chartsheet.custom import CustomChartsheetViews
from openpyxl.chartsheet.properties import ChartsheetProperties
from openpyxl.chartsheet.protection import ChartsheetProtection
from openpyxl.chartsheet.publish import WebPublishItems
from openpyxl.chartsheet.relation import DrawingHF, SheetBackgroundPicture
from openpyxl.chartsheet.views import ChartsheetViewList
from openpyxl.descriptors.base import Alias, Set, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.workbook.child import _WorkbookChild
from openpyxl.workbook.workbook import Workbook
from openpyxl.worksheet.drawing import Drawing
from openpyxl.worksheet.header_footer import HeaderFooter as _HeaderFooter
from openpyxl.worksheet.page import PageMargins, PrintPageSetup
from openpyxl.xml.functions import Element

class Chartsheet(_WorkbookChild, Serialisable):
    tagname: ClassVar[str]
    mime_type: str
    sheetPr: Typed[ChartsheetProperties, Literal[True]]
    sheetViews: Typed[ChartsheetViewList, Literal[False]]
    sheetProtection: Typed[ChartsheetProtection, Literal[True]]
    customSheetViews: Typed[CustomChartsheetViews, Literal[True]]
    pageMargins: Typed[PageMargins, Literal[True]]
    pageSetup: Typed[PrintPageSetup, Literal[True]]
    drawing: Typed[Drawing, Literal[True]]
    drawingHF: Typed[DrawingHF, Literal[True]]
    picture: Typed[SheetBackgroundPicture, Literal[True]]
    webPublishItems: Typed[WebPublishItems, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    sheet_state: Set[_VisibilityType]
    headerFooter: Typed[_HeaderFooter, Literal[False]]
    HeaderFooter: Alias  # type: ignore[assignment] # Different from parent class
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        sheetPr: ChartsheetProperties | None = None,
        sheetViews: ChartsheetViewList | None = None,
        sheetProtection: ChartsheetProtection | None = None,
        customSheetViews: CustomChartsheetViews | None = None,
        pageMargins: PageMargins | None = None,
        pageSetup: PrintPageSetup | None = None,
        headerFooter: _HeaderFooter | None = None,
        drawing: Unused = None,
        drawingHF: DrawingHF | None = None,
        picture: SheetBackgroundPicture | None = None,
        webPublishItems: WebPublishItems | None = None,
        extLst: Unused = None,
        parent: Workbook | None = None,
        title: str | _Decodable | None = "",
        sheet_state: _VisibilityType = "visible",
    ) -> None: ...
    def add_chart(self, chart) -> None: ...
    def to_tree(self) -> Element: ...  # type: ignore[override]



import threading
from _typeshed import ReadableBuffer
from collections.abc import Callable
from types import TracebackType
from typing_extensions import Self

from serial import Serial

class Protocol:
    def connection_made(self, transport: ReaderThread) -> None: ...
    def data_received(self, data: bytes) -> None: ...
    def connection_lost(self, exc: BaseException | None) -> None: ...

class Packetizer(Protocol):
    TERMINATOR: bytes
    buffer: bytearray
    transport: ReaderThread | None
    def handle_packet(self, packet: bytes) -> None: ...

class FramedPacket(Protocol):
    START: bytes
    STOP: bytes
    packet: bytearray
    in_packet: bool
    transport: ReaderThread | None
    def handle_packet(self, packet: bytes) -> None: ...
    def handle_out_of_packet_data(self, data: bytes) -> None: ...

class LineReader(Packetizer):
    ENCODING: str
    UNICODE_HANDLING: str
    def handle_line(self, line: str) -> None: ...
    def write_line(self, text: str) -> None: ...

class ReaderThread(threading.Thread):
    serial: Serial
    protocol_factory: Callable[[], Protocol]
    alive: bool
    protocol: Protocol
    def __init__(self, serial_instance: Serial, protocol_factory: Callable[[], Protocol]) -> None: ...
    def stop(self) -> None: ...
    def write(self, data: ReadableBuffer) -> int: ...
    def close(self) -> None: ...
    def connect(self) -> tuple[Self, Protocol]: ...
    def __enter__(self) -> Protocol: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None, /
    ) -> None: ...
